@@ -1,6 +1,6 @@
 /* gcompris - algebra.c
  *
- * Time-stamp: <2002/04/17 00:25:30 bruno>
+ * Time-stamp: <2004/03/10 23:15:42 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -23,19 +23,20 @@
 
 #include "gcompris/gcompris.h"
 
-#define PLUSSIGNFILE '+'
-#define MINUSSIGNFILE '-'
-#define BYSIGNFILE '*'
+#define TIMER_X 400
+#define TIMER_Y 140
+
+#define PLUSSIGNFILE   '+'
+#define MINUSSIGNFILE  '-'
+#define BYSIGNFILE     'x'
 #define DIVIDESIGNFILE ':'
 
 /* Some constants for the numbers layout */
 #define NUMBERSWIDTH       45
-#define NUMBERSHEIGHT      64
-#define NUMBERSSPACEHEIGHT NUMBERSHEIGHT
 
 #define SOUNDLISTFILE PACKAGE
 
-gboolean	 board_paused = TRUE;
+static gboolean	 board_paused = TRUE;
 static int	 leavenow;
 static int	 gamewon;
 static void	 game_won(void);
@@ -48,17 +49,13 @@ static GcomprisBoard *gcomprisBoard = NULL;
 
 static char *expected_result = NULL;
 
-/* Default layout for the algebra operation */
-static gboolean vertical_layout = FALSE;
-
-static gboolean operation_done[10];
+static gboolean operation_done[11];
 
 typedef struct {
   guint index;
   GnomeCanvasItem *item;
   GnomeCanvasItem *focus_item;
   GnomeCanvasItem *bad_item;
-  GdkPixbuf *image;
   char *next;  /* Help : Should point to a ToBeFoundItem but don't know how to recurse on it */
   char *previous;
   char value;
@@ -68,14 +65,14 @@ static ToBeFoundItem *currentToBeFoundItem = NULL;
 
 static GnomeCanvasGroup *boardRootItem = NULL;
 
-static char currentOperation = PLUSSIGNFILE;
+static char currentOperation[2];
 
 static void		 start_board (GcomprisBoard *agcomprisBoard);
 static void		 pause_board (gboolean pause);
 static void		 end_board (void);
 static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
 static void		 set_level (guint level);
-gint			 key_press(guint keyval);
+static gint		 key_press(guint keyval);
 static void		 process_ok(void);
 
 static GnomeCanvasItem	*algebra_create_item(GnomeCanvasGroup *parent);
@@ -93,12 +90,12 @@ static void		 set_focus_item(ToBeFoundItem *toBeFoundItem, gboolean status);
 static void		 init_operation(void);
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
    N_("Learning Clock"),
-   N_("Learn how to read the time"),
+   N_("Learn how to tell time"),
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL,
    NULL,
@@ -136,10 +133,11 @@ static void pause_board (gboolean pause)
   if(gcomprisBoard==NULL)
     return;
 
+  /* Make the timer follow our pause status */
+  gcompris_timer_pause(pause);
+
   if(gamewon == TRUE && pause == FALSE) /* the game is won */
-    {
       algebra_next_level();
-    }
 
   if(leavenow == TRUE && pause == FALSE)
     board_finished(BOARD_FINISHED_RANDOM);
@@ -156,7 +154,7 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "images/scenery2_background.jpg");
 
       /* set initial values for this level */
       gcomprisBoard->level=1;
@@ -170,24 +168,19 @@ static void start_board (GcomprisBoard *agcomprisBoard)
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_OK);
 
       /* The mode defines the operation */
-      /* If added with the sign | then the vertical layout is selected */
 
       /* Default mode */
       if(!gcomprisBoard->mode)
-	currentOperation=PLUSSIGNFILE;
+	currentOperation[0]=PLUSSIGNFILE;
       else if(g_strncasecmp(gcomprisBoard->mode, "+", 1)==0)
-	currentOperation=PLUSSIGNFILE;
+	currentOperation[0]=PLUSSIGNFILE;
       else if(g_strncasecmp(gcomprisBoard->mode, "-", 1)==0)
-	currentOperation=MINUSSIGNFILE;
+	currentOperation[0]=MINUSSIGNFILE;
       else if(g_strncasecmp(gcomprisBoard->mode, "*", 1)==0)
-	currentOperation=BYSIGNFILE;
+	currentOperation[0]=BYSIGNFILE;
       else if(g_strncasecmp(gcomprisBoard->mode, "/", 1)==0)
-	currentOperation=DIVIDESIGNFILE;
-      
-      if(g_strncasecmp(gcomprisBoard->mode+1, "|", 1)==0)
-	 vertical_layout=TRUE;
-      else
-	vertical_layout=FALSE;
+	currentOperation[0]=DIVIDESIGNFILE;
+      currentOperation[1]='\0';
 
       init_operation();
       algebra_next_level();
@@ -224,7 +217,7 @@ set_level (guint level)
     }
 }
 
-gint key_press(guint keyval)
+static gint key_press(guint keyval)
 {
   guint c;
   gboolean stop = FALSE;
@@ -304,6 +297,8 @@ gint key_press(guint keyval)
 	  set_focus_item(currentToBeFoundItem, TRUE);
 	}
       return TRUE;
+    case GDK_Delete:
+    case GDK_BackSpace:
     case GDK_Left:
       if(currentToBeFoundItem->next!=NULL)
 	{
@@ -319,10 +314,13 @@ gint key_press(guint keyval)
   if(currentToBeFoundItem!=NULL &&
      keyval>=GDK_0 && keyval<=GDK_9)
     {
-      currentToBeFoundItem->image = gcompris_load_number_pixmap(c);
+      char number[2];
+      number[1] = '\0';
+      number[0] = (char)c;
+
       currentToBeFoundItem->value = c;
       gnome_canvas_item_set (currentToBeFoundItem->item,
-			     "pixbuf", currentToBeFoundItem->image,
+			     "text", (char *)&number,
 			     NULL);
 
       /* Not a failure (yet) */
@@ -334,20 +332,10 @@ gint key_press(guint keyval)
       /* Move the focus to the next appropriate digit */
       while(!stop)
 	{
-	  if(vertical_layout)
-	    {
-	      if(currentToBeFoundItem->next!=NULL)
-		currentToBeFoundItem = (ToBeFoundItem *)currentToBeFoundItem->next;
-	      else
-		stop = TRUE;
-	    }
+	  if(currentToBeFoundItem->previous!=NULL)
+	    currentToBeFoundItem = (ToBeFoundItem *)currentToBeFoundItem->previous;
 	  else
-	    {
-	      if(currentToBeFoundItem->previous!=NULL)
-		currentToBeFoundItem = (ToBeFoundItem *)currentToBeFoundItem->previous;
-	      else
-		stop = TRUE;
-	    }
+	    stop = TRUE;
 
 	  if(currentToBeFoundItem->in_error==TRUE ||
 	     currentToBeFoundItem->value=='?')
@@ -361,7 +349,7 @@ gint key_press(guint keyval)
 
 
 
-gboolean
+static gboolean
 is_our_board (GcomprisBoard *gcomprisBoard)
 {
   if (gcomprisBoard)
@@ -409,7 +397,8 @@ static void algebra_next_level()
 							    NULL));
 
   maxtime = 20;
-  gcompris_timer_display(150, 140, GCOMPRIS_TIMER_BALLOON, maxtime, timer_end);
+  gcompris_timer_display(TIMER_X, TIMER_Y, 
+			 GCOMPRIS_TIMER_BALLOON, maxtime, timer_end);
 
   /* Try the next level */
   algebra_create_item(boardRootItem);
@@ -473,7 +462,6 @@ static void display_operand(GnomeCanvasGroup *parent,
 			    char *operand_str,
 			    gboolean masked)
 {
-  GdkPixbuf *algebra_pixmap = NULL;
   GnomeCanvasItem *item, *focus_item = NULL, *bad_item = NULL;
   int i;
   ToBeFoundItem *toBeFoundItem=NULL;
@@ -481,60 +469,45 @@ static void display_operand(GnomeCanvasGroup *parent,
 
   for(i=strlen(operand_str)-1; i>=0; i--)
     {
-      if(masked) 
-	{
-	  algebra_pixmap = gcompris_load_number_pixmap('?');
-	} 
-      else 
-	{
-	  algebra_pixmap = gcompris_load_number_pixmap(operand_str[i]);
-	}
+      char operand[2] = "?";
 
+      if(!masked)
+	operand[0] = operand_str[i];
+      
       item = gnome_canvas_item_new (parent,
-				    gnome_canvas_pixbuf_get_type (),
-				    "pixbuf", algebra_pixmap, 
-				    "x", x_align-((strlen(operand_str)-i)*NUMBERSWIDTH)
-				    +(NUMBERSWIDTH-gdk_pixbuf_get_width(algebra_pixmap))/2-
-				    gdk_pixbuf_get_width(algebra_pixmap)/2,
-				    "y", y - gdk_pixbuf_get_height(algebra_pixmap)/2,
-				    "width", (double) gdk_pixbuf_get_width(algebra_pixmap),
-				    "height", (double) gdk_pixbuf_get_height(algebra_pixmap),
+				    gnome_canvas_text_get_type (),
+				    "text", &operand, 
+				    "font", gcompris_skin_font_board_huge_bold,
+				    "anchor", GTK_ANCHOR_CENTER,
+				    "x", x_align-((strlen(operand_str)-i)*NUMBERSWIDTH),
+				    "y", y,
+				    "fill_color_rgba", 0x2c2cFFFF,
 				    NULL);
-      gdk_pixbuf_unref(algebra_pixmap);
       item_list = g_list_append (item_list, item);
 
       if(masked) 
 	{
 	  focus_item = gnome_canvas_item_new (parent,
-					      gnome_canvas_rect_get_type (),
-					      "x1",  x_align-((strlen(operand_str)-i)*NUMBERSWIDTH)
-					      +(NUMBERSWIDTH-gdk_pixbuf_get_width(algebra_pixmap))/2 + 5
-					      - (gdk_pixbuf_get_width(algebra_pixmap))/2,
-					      "y1", (double)y + (gdk_pixbuf_get_height(algebra_pixmap))/2,
-					      "x2", (double)x_align-((strlen(operand_str)-i)*NUMBERSWIDTH)
-					      +(NUMBERSWIDTH-gdk_pixbuf_get_width(algebra_pixmap))/2 + 
-					      gdk_pixbuf_get_width(algebra_pixmap) - 5
-					      - (gdk_pixbuf_get_width(algebra_pixmap))/2,
-					      "y2", (double)y + (gdk_pixbuf_get_height(algebra_pixmap))/2 + 5,
-					      "fill_color", "black",
-					      "outline_color", "blue",
-					      "width_units", (double)2,
+					      gnome_canvas_text_get_type (),
+					      "text", "_",
+					      "font", gcompris_skin_font_board_huge_bold,
+					      "anchor", GTK_ANCHOR_CENTER,
+					      "x", x_align-((strlen(operand_str)-i)*NUMBERSWIDTH),
+					      "y", y,
+					      "fill_color_rgba", 0x00ae00FF,
 					      NULL);
 
 	  item_list = g_list_append (item_list, focus_item);
 
-	  algebra_pixmap = gcompris_load_pixmap("gcompris/buttons/bad.png");
 	  bad_item = gnome_canvas_item_new (parent,
-					    gnome_canvas_pixbuf_get_type (),
-					    "pixbuf", algebra_pixmap, 
-					    "x", x_align-((strlen(operand_str)-i)*NUMBERSWIDTH)
-					    +(NUMBERSWIDTH-gdk_pixbuf_get_width(algebra_pixmap))/2 
-					    -gdk_pixbuf_get_width(algebra_pixmap)/2,
-					    "y", y -gdk_pixbuf_get_height(algebra_pixmap)/2,
-					    "width", (double) gdk_pixbuf_get_width(algebra_pixmap),
-					    "height", (double) gdk_pixbuf_get_height(algebra_pixmap),
+					    gnome_canvas_text_get_type (),
+					    "text", "/",
+					    "font", gcompris_skin_font_board_huge_bold,
+					    "anchor", GTK_ANCHOR_CENTER,
+					    "x", x_align-((strlen(operand_str)-i)*NUMBERSWIDTH),
+					    "y", y,
+					    "fill_color_rgba", 0xFF0000FF,
 					    NULL);
-	  gdk_pixbuf_unref(algebra_pixmap);
 	  gnome_canvas_item_hide(bad_item);
 
 	  item_list = g_list_append (item_list, bad_item);
@@ -547,7 +520,6 @@ static void display_operand(GnomeCanvasGroup *parent,
 	  toBeFoundItem->bad_item=bad_item;
 	  toBeFoundItem->index=i;
 	  toBeFoundItem->value='?';
-	  toBeFoundItem->image=algebra_pixmap;
 	  toBeFoundItem->item=item;
 	  toBeFoundItem->focus_item=focus_item;
 	  toBeFoundItem->previous=(char *)previousToBeFoundItem;
@@ -559,8 +531,8 @@ static void display_operand(GnomeCanvasGroup *parent,
 
 	  previousToBeFoundItem=toBeFoundItem;
 
-	  /* Init the focussed item on the right digit depending on vertical_layout */
-	  if(i==(vertical_layout?strlen(operand_str)-1:0))
+	  /* Init the focussed item on the right digit */
+	  if(i==0)
 	    {
 	      currentToBeFoundItem=toBeFoundItem;
 	      set_focus_item(toBeFoundItem, TRUE);
@@ -578,7 +550,6 @@ static void display_operand(GnomeCanvasGroup *parent,
 
 static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
 {
-  GdkPixbuf *algebra_pixmap = NULL;
   GnomeCanvasItem *item;
   guint first_operand, second_operand;
   char *first_operand_str = NULL;
@@ -592,7 +563,7 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
 
 
   /* Some globals for this board layout */
-  y_firstline = gcomprisBoard->height/3;
+  y_firstline = gcomprisBoard->height/5;
 
 
   get_random_number(&first_operand, &second_operand);
@@ -603,76 +574,44 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
   /* Calc the longuest value */
   longuest = MAX(strlen(first_operand_str), strlen(second_operand_str));
 
-  if(vertical_layout)
-    /* Vertical layout : Warning x_align is the right assigned value for display_operand */
-    x_align = gcomprisBoard->width - (gcomprisBoard->width - (longuest*NUMBERSWIDTH))/2 + 150;
-  else
-    x_align = (gcomprisBoard->width - (longuest*3*NUMBERSWIDTH))/2 + NUMBERSWIDTH*(strlen(first_operand_str)) + 150;
+  x_align = (gcomprisBoard->width - (longuest*3*NUMBERSWIDTH))/2 + NUMBERSWIDTH*(strlen(first_operand_str)) - 200;
 
   /* First operand */
   display_operand(parent, x_align, y_firstline, first_operand_str, FALSE);
 
   /* Second operand */
-  if(vertical_layout)
-      display_operand(parent, x_align, y_firstline + NUMBERSSPACEHEIGHT, second_operand_str, FALSE);
-  else
-      display_operand(parent, x_align + NUMBERSWIDTH*(strlen(second_operand_str)+1),
-		      y_firstline, second_operand_str, FALSE);
+  display_operand(parent, x_align + NUMBERSWIDTH*(strlen(second_operand_str)+1),
+		  y_firstline, second_operand_str, FALSE);
 
   /* Display the operator */
-  algebra_pixmap = gcompris_load_operation_pixmap(currentOperation);
-  if(vertical_layout)
-    {
-      x=(double) gcomprisBoard->width - x_align - NUMBERSWIDTH - 20;
-      y=(double) y_firstline + NUMBERSSPACEHEIGHT;
-    }
-  else
-    {
-      x=(double) x_align;
-      y=(double) y_firstline;
-    }
+  x=(double) x_align;
+  y=(double) y_firstline;
   item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf", algebra_pixmap, 
-				"x", x -  gdk_pixbuf_get_width(algebra_pixmap)/2,
-				"y", y -  gdk_pixbuf_get_height(algebra_pixmap)/2,
-				"width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				"height", (double) gdk_pixbuf_get_height(algebra_pixmap),
+				gnome_canvas_text_get_type (),
+				"text", currentOperation, 
+				"font", gcompris_skin_font_board_huge_bold,
+				"x", x,
+				"y", y,
+				"anchor", GTK_ANCHOR_CENTER,
+				"fill_color_rgba", 0xFF3333FF,
 				NULL);
   item_list = g_list_append (item_list, item);
 
-  /* Now the separator line or the equal sign*/
-  if(vertical_layout)
-    {
-      item = gnome_canvas_item_new (parent,
-				    gnome_canvas_rect_get_type (),
-				    "x1", (double) gcomprisBoard->width - x_align - NUMBERSWIDTH - 20,
-				    "y1", (double) y_firstline + NUMBERSSPACEHEIGHT + NUMBERSSPACEHEIGHT/2,
-				    "x2", (double) x_align+(NUMBERSWIDTH-gdk_pixbuf_get_width(algebra_pixmap))/2,
-				    "y2", (double)  y_firstline + NUMBERSSPACEHEIGHT + NUMBERSSPACEHEIGHT/2 + 5,
-				    "fill_color", "blue",
-				    "outline_color", "green",
-				    "width_units", (double)1,
-				    NULL);
-    }
-  else
-    {
-      algebra_pixmap = gcompris_load_operation_pixmap('=');
-      item = gnome_canvas_item_new (parent,
-				    gnome_canvas_pixbuf_get_type (),
-				    "pixbuf", algebra_pixmap, 
-				    "x", (double) x_align + NUMBERSWIDTH*(strlen(second_operand_str)+1)
-				    - gdk_pixbuf_get_width(algebra_pixmap)/2,
-				    "y", (double) y_firstline - gdk_pixbuf_get_height(algebra_pixmap)/2,
-				    "width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				    "height", (double) gdk_pixbuf_get_height(algebra_pixmap),
-				    NULL);
-    }
+  /* Now the equal sign*/
+  item = gnome_canvas_item_new (parent,
+				gnome_canvas_text_get_type (),
+				"text", "=", 
+				"font", gcompris_skin_font_board_huge_bold,
+				"x", x_align + NUMBERSWIDTH*(strlen(second_operand_str)+1),
+				"y", y,
+				"anchor", GTK_ANCHOR_CENTER,
+				"fill_color_rgba", 0xFF3333FF,
+				NULL);
   item_list = g_list_append (item_list, item);
 
   /* Display the empty area */
   g_free(expected_result);
-  switch(currentOperation)
+  switch(currentOperation[0])
     {
     case PLUSSIGNFILE:
       expected_result = g_strdup_printf("%d", first_operand+second_operand);
@@ -693,13 +632,10 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
     default:
       g_error("Bad Operation");
     }
-  if(vertical_layout)
-    display_operand(parent, x_align, y_firstline + NUMBERSSPACEHEIGHT*2 + 10, expected_result, TRUE);
-  else
-    display_operand(parent, x_align + NUMBERSWIDTH*(strlen(second_operand_str)+
-						    strlen(expected_result)+2),
-		    y_firstline, expected_result, TRUE);
-
+  display_operand(parent, x_align + NUMBERSWIDTH*(strlen(second_operand_str)+
+						  strlen(expected_result)+2),
+		  y_firstline, expected_result, TRUE);
+  
 
   gcompris_play_ogg(first_operand_str, audioOperand , second_operand_str, "equal", NULL);
 
@@ -713,17 +649,11 @@ static void set_focus_item(ToBeFoundItem *toBeFoundItem, gboolean status)
 {
   if(status)
     {
-      gnome_canvas_item_set (toBeFoundItem->focus_item,
-			     "fill_color", "blue",
-			     "outline_color", "green",
-			     NULL);
+      gnome_canvas_item_show (toBeFoundItem->focus_item);
     }
   else
     {
-      gnome_canvas_item_set (toBeFoundItem->focus_item,
-			     "fill_color", NULL,
-			     "outline_color", NULL,
-			     NULL);
+      gnome_canvas_item_hide (toBeFoundItem->focus_item);
     }
 }
 
@@ -747,11 +677,8 @@ static void process_ok()
 	{
 	  gnome_canvas_item_show(currentToBeFoundItem->bad_item);
 	  currentToBeFoundItem->in_error = TRUE;
-	  /* Depending on vertical_layout, remember the appropriate digit to focus next */
-	  if(vertical_layout && hasfail==NULL)
-	    hasfail=currentToBeFoundItem;
-	  else
-	    hasfail=currentToBeFoundItem;
+	  /* remember the appropriate digit to focus next */
+	  hasfail=currentToBeFoundItem;
 	}
       currentToBeFoundItem=(ToBeFoundItem *)currentToBeFoundItem->next;
     }
@@ -795,7 +722,7 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
 
       currentToBeFoundItem=toBeFoundItem;
 
-      gcompris_play_sound (SOUNDLISTFILE, "gobble");
+      gcompris_play_ogg ("gobble", NULL);
       break;
       
     default:
@@ -814,15 +741,14 @@ static gint get_operand()
   gint i = (1+rand()%10);
   
   // Get the next free slot
-  while(operation_done[i]==TRUE && j>0)
+  while(operation_done[i]==TRUE && j>=0)
     {
       j--;
       i++;
-      if(i>9)
-	i=0;
+      if(i>10)
+	i=1;
     }
   operation_done[i]=TRUE;
-
   return i;
 }
 
@@ -830,7 +756,7 @@ static void init_operation()
 {
   int i;
 
-  for(i=0; i<10; i++)
+  for(i=1; i<=10; i++)
     operation_done[i] = FALSE;
 }
 
@@ -839,7 +765,7 @@ static void get_random_number(guint *first_operand, guint *second_operand)
 {
   guint min, max;
 
-  switch(currentOperation)
+  switch(currentOperation[0])
     {
     case PLUSSIGNFILE:
       *first_operand = get_operand();
