@@ -1,6 +1,6 @@
 /* gcompris - clickgame.c
  *
- * Time-stamp: <2001/09/12 23:42:27 bruno>
+ * Time-stamp: <2004/06/12 23:31:34 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -19,7 +19,11 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef __APPLE__
+#   include <sys/types.h>
+#endif
 #include <dirent.h>
+#include <string.h>
 
 #include "gcompris/gcompris.h"
 #include "gcompris/pixbuf_util.h"
@@ -28,6 +32,8 @@
 
 #define SOUNDLISTFILE PACKAGE
 
+static gboolean board_paused = TRUE;
+
 static GList *item_list = NULL;
 static GList *item2del_list = NULL;
 
@@ -52,7 +58,7 @@ static void	 clickgame_pause (gboolean pause);
 static void	 clickgame_end (void);
 static gboolean	 clickgame_is_our_board (GcomprisBoard *gcomprisBoard);
 static void	 clickgame_set_level (guint level);
-static void	 clickgame_help(void);
+static void	 clickgame_config(void);
 
 static FishItem *clickgame_create_item(GnomeCanvasGroup *parent);
 static gint	 clickgame_drop_items (GtkWidget *widget, gpointer data);
@@ -62,20 +68,23 @@ static void	 clickgame_destroy_item(FishItem *fishitem);
 static void	 clickgame_destroy_items(void);
 static void	 clickgame_destroy_all_items(void);
 static void	 setup_item(FishItem *fishitem);
-static void	 load_random_pixmap();
+static void	 load_random_pixmap(void);
 static void	 clickgame_next_level(void);
 
+static int gamewon;
+static void	 game_won(void);
+
 static  guint32              fallSpeed = 0;
 static  double               speed = 0.0;
 static  double               imageZoom = 0.0;
 
 /* Description of this plugin */
-BoardPlugin clickgame_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
    N_("Click On Me"),
-   N_("Left-Click with the mouse on all swimming fishes before they leave the fishtank"),
+   N_("Left-Click with the mouse on all swimming fish before they leave the fishtank"),
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL,
    NULL,
@@ -88,7 +97,8 @@ BoardPlugin clickgame_bp =
    NULL,
    NULL,
    clickgame_set_level,
-   clickgame_help,
+   clickgame_config,
+   NULL
 };
 
 /*
@@ -101,7 +111,7 @@ BoardPlugin clickgame_bp =
 BoardPlugin 
 *get_bplugin_info(void)
 {
-   return &clickgame_bp;
+   return &menu_bp;
 }
 
 /*
@@ -113,6 +123,11 @@ static void clickgame_pause (gboolean pause)
   if(gcomprisBoard==NULL)
     return;
 
+  if(gamewon == TRUE && pause == FALSE) /* the game is won */
+    {
+      game_won();
+    }
+
   if(pause)
     {
       if (dummy_id) {
@@ -141,6 +156,8 @@ static void clickgame_pause (gboolean pause)
 	animate_id = gtk_timeout_add (200, (GtkFunction) clickgame_animate_items, NULL);
       }
     }
+
+  board_paused = pause;
 }
 
 /*
@@ -148,29 +165,29 @@ static void clickgame_pause (gboolean pause)
 static void clickgame_start (GcomprisBoard *agcomprisBoard)
 {
   gint i;
-  char *str;
+  gchar *str;
   gchar *filename;
-  struct dirent **namelist = NULL;
-  int namelistlength = 0;
+  DIR *dir;
+  struct dirent *one_dirent;
 
   if(agcomprisBoard!=NULL)
     {
       gcomprisBoard = agcomprisBoard;
 
       /* Load the Pixpmaps directory file names */
-      filename = g_strdup_printf("%s/%s", PACKAGE_DATA_DIR, gcomprisBoard->boarddir);
-      namelistlength = scandir(filename,
-			       &namelist, 0, alphasort);
+      filename = g_strdup_printf("%s/%s", PACKAGE_DATA_DIR, gcomprisBoard->boarddir);      
+      dir = opendir(filename);
+      
+      if (!dir)
+	g_error (_("Couldn't open dir: %s"),filename);
 
       g_free(filename);
 
-      if (namelistlength < 0)
-	g_error (_("Couldn't open pixmap dir: %s"), filename);
+      while((one_dirent = readdir(dir)) != NULL) {
 
-      /* Fill up the pixmap list */
-      for(i=2; i<namelistlength; i++)
-	{
-	  str = g_strdup_printf("%s/%s", gcomprisBoard->boarddir, namelist[i]->d_name);
+	if (one_dirent->d_name[0] != '.') {
+
+	  str = g_strdup_printf("%s/%s", gcomprisBoard->boarddir, one_dirent->d_name);
 	  str[strlen(str)-5]='x';
 
 	  if(g_list_find_custom(pixmaplist, str, (GCompareFunc) strcmp) == NULL)
@@ -178,11 +195,19 @@ static void clickgame_start (GcomprisBoard *agcomprisBoard)
 	    pixmaplist = g_list_append (pixmaplist, str);
 	  }
 	}
+      }
+      closedir(dir);
+
 
       /* set initial values for this level */
       gcomprisBoard->level = 1;
       gcomprisBoard->maxlevel=6;
-      gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_HELP);
+      gcomprisBoard->number_of_sublevel=10; /* Go to next level after this number of 'play' */
+      gcompris_score_start(SCORESTYLE_NOTE, 
+			   gcomprisBoard->width - 220, 
+			   gcomprisBoard->height - 50, 
+			   gcomprisBoard->number_of_sublevel);
+      gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
       clickgame_next_level();
 
@@ -199,13 +224,11 @@ clickgame_end ()
   if(gcomprisBoard!=NULL)
     {
       clickgame_pause(TRUE);
+      gcompris_score_end();
       clickgame_destroy_all_items();
       gcomprisBoard->level = 1;       // Restart this game to zero
-
-      /* reset the timer */
-      gcompris_bar_set_timer(0);
-
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -219,16 +242,15 @@ clickgame_set_level (guint level)
     }
 }
 
-gboolean
+static gboolean
 clickgame_is_our_board (GcomprisBoard *gcomprisBoard)
 {
-
   if (gcomprisBoard)
     {
       if(g_strcasecmp(gcomprisBoard->type, "clickgame")==0)
 	{
 	  /* Set the plugin entry */
-	  gcomprisBoard->plugin=&clickgame_bp;
+	  gcomprisBoard->plugin=&menu_bp;
 
 	  return TRUE;
 	}
@@ -237,31 +259,11 @@ clickgame_is_our_board (GcomprisBoard *gcomprisBoard)
 }
 
 static void
-clickgame_help ()
+clickgame_config ()
 {
   if(gcomprisBoard!=NULL)
     {
       clickgame_pause(TRUE);
-      gcompris_help_start(N_("Practice the computer mouse"), 
-			  N_("\
-Goal:\n\
-In this game, the children will discover the computer mouse\n\
-The goal is just to click with the left mouse button on the moving fish.\n\
-\n\
-Credits:\n\
-Fish are taken from the Unix utility xfishtank.\n\
-Images are taken from the National Undersearch Research Program (NURP) Collection\n\
-at http://www.photolib.noaa.gov\n\
-All images credit bellongs to the OAR/National Undersea Research Program (NURP)\n\
-Except for the first image that also credits the University of North Carolina at Wilmington\n\
-In the gcompris level order, images information is:\n\
-Location: Tropical Atlantic Ocean, Florida Keys (image nur00523)\n\
-Photographer: D. Kesling\n\
-Location: Florida Keys (image nur03006)\n\
-Location: Unknown (image nur03505)\n\
-Location: Unknown (image nur03010)\n\
-Location: Unknown (image nur03011)\n\
-Location: Unknown (image nur03013)\n"));
     }
 }
 
@@ -282,7 +284,7 @@ static void clickgame_next_level()
       break;
     case 2:
       gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), 
-			      "gcompris/nur03006.jpg");
+      			      "gcompris/nur03006.jpg");
       break;
     case 3:
       gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), 
@@ -303,17 +305,13 @@ static void clickgame_next_level()
 
   gcompris_bar_set_level(gcomprisBoard);
 
-  clickgame_destroy_all_items();
-
   /* Try the next level */
   speed=100+(40/(gcomprisBoard->level));
   fallSpeed=5000-gcomprisBoard->level*200;
-  gcomprisBoard->number_of_sublevel=10;
   /* Make the images tend to 0.5 ratio */
   imageZoom=0.5+(0.5/(gcomprisBoard->level));
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
   gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcompris_score_set(gcomprisBoard->sublevel);
 
 }
 
@@ -354,14 +352,14 @@ static void clickgame_move_item(FishItem *fishitem)
     {
       if(x1>gcomprisBoard->width) {
 	item2del_list = g_list_append (item2del_list, fishitem);
-	gcompris_play_sound (SOUNDLISTFILE, "crash");
+	gcompris_play_ogg ("crash", NULL);
       }
     }
   else
     {
       if(x2<0) {
 	item2del_list = g_list_append (item2del_list, fishitem);
-	gcompris_play_sound (SOUNDLISTFILE, "crash");
+	gcompris_play_ogg ("crash", NULL);
       }
     }
 
@@ -395,13 +393,32 @@ static void clickgame_destroy_items()
 static void clickgame_destroy_all_items()
 {
   FishItem *fishitem;
+  int i;
 
   while(g_list_length(item_list)>0) 
     {
       fishitem = g_list_nth_data(item_list, 0);
       clickgame_destroy_item(fishitem);
     }
+  if(item_list) {
+    g_list_free(item_list);
+    item_list = NULL;
+  }
 
+  i=g_list_length(pixmaplist);
+  for(i=0; i<g_list_length(pixmaplist); i++) {
+    g_free(g_list_nth_data(item_list, i));
+  }
+
+  if(pixmaplist) {
+    g_list_free(pixmaplist);
+    pixmaplist = NULL;
+  }
+
+  if(imagelist) {
+    g_list_free(imagelist);
+    imagelist = NULL;
+  }
 }
 
 /*
@@ -463,10 +480,12 @@ static FishItem *clickgame_create_item(GnomeCanvasGroup *parent)
   if(fishitem->speed<0)
     {
       x = (double) gcomprisBoard->width;
+      fishitem->speed=MIN(fishitem->speed, -1);
     }
   else 
     {
       x = (double) -gdk_pixbuf_get_width(pixmap)*imageZoom;
+      fishitem->speed=MAX(fishitem->speed, 1);
     }
 
   rootitem = \
@@ -538,6 +557,24 @@ static gint clickgame_drop_items (GtkWidget *widget, gpointer data)
   return (FALSE);
 }
 
+/* ==================================== */
+static void game_won()
+{
+  gcomprisBoard->sublevel++;
+
+  if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
+    /* Try the next level */
+    gcomprisBoard->sublevel=0;
+    gcomprisBoard->level++;
+    if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+      board_finished(BOARD_FINISHED_RANDOM);
+      return;
+    }
+    gcompris_play_ogg ("bonus", NULL);
+  }
+  clickgame_next_level();
+}
+
 static gint
 item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
 {
@@ -547,7 +584,10 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
    static int dragging;
    double item_x, item_y;
 
-   if(!get_board_playing())
+   if(!gcomprisBoard)
+     return FALSE;
+
+   if(board_paused)
      return FALSE;
 
    item_x = event->button.x;
@@ -577,17 +617,15 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
            else 
              {
 	       clickgame_destroy_item(fishitem);
-	       gcompris_play_sound (SOUNDLISTFILE, "gobble");
-
+	       gcompris_play_ogg ("gobble", NULL);
+	       
 	       gcomprisBoard->sublevel++;
-	       gcompris_bar_set_timer(gcomprisBoard->sublevel);
+	       gcompris_score_set(gcomprisBoard->sublevel);
+	       
 	       if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
-		 /* Try the next level */
-		 gcomprisBoard->level++;
-		 if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-		   gcomprisBoard->level=gcomprisBoard->maxlevel;
-		 clickgame_next_level();
-		 gcompris_play_sound (SOUNDLISTFILE, "bonus");
+		 gamewon = TRUE;
+		 clickgame_destroy_all_items();
+		 gcompris_display_bonus(gamewon, BONUS_FLOWER);
 		 return FALSE;
 	       }
 	       /* Drop a new item now to speed up the game */
@@ -607,6 +645,32 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
              }
            break;
 
+         case 3:
+	   /* Speed up the fish */
+	   if(fishitem->speed>0)
+	     fishitem->speed = MAX(fishitem->speed+1 , 5);
+	   else
+	     fishitem->speed = MIN(fishitem->speed-1, -5);
+	   break;
+
+         case 2:
+	   /* Slow the fish */
+	   if(fishitem->speed>0)
+	     fishitem->speed = MAX(fishitem->speed-1, 1);
+	   else
+	     fishitem->speed = MIN(fishitem->speed+1, -1);
+	   break;
+
+         case 4:
+	   /* fish up */
+	   gnome_canvas_item_move(item, 0.0, -3.0);
+	   break;
+
+         case 5:
+	   /* fish down */
+	   gnome_canvas_item_move(item, 0.0, 3.0);
+	   break;
+
          default:
            break;
          }
@@ -643,9 +707,9 @@ static void
 setup_item(FishItem *fishitem)
 {
   if(fishitem)
-    gtk_signal_connect(GTK_OBJECT(fishitem->rootitem), "event",
-		       (GtkSignalFunc) item_event,
-		       fishitem);
+      gtk_signal_connect(GTK_OBJECT(fishitem->rootitem), "event",
+			 (GtkSignalFunc) item_event,
+			 fishitem);
 }
 
 /*
@@ -656,13 +720,12 @@ static void load_random_pixmap()
 {
   GdkPixbuf *pixmap;
   int i, j;
-  char *str = NULL;
+  gchar *str = NULL;
   gboolean cont = TRUE;
 
   if(g_list_length(pixmaplist)<=0)
     return;
 
-
   i=rand()%(g_list_length(pixmaplist));
 
   str = (char *)g_list_nth_data(pixmaplist, i);
@@ -682,15 +745,19 @@ static void load_random_pixmap()
 
       filename = g_strdup_printf("%s/%s", PACKAGE_DATA_DIR, str);
 
-      if (!g_file_exists (filename))
+      if (!g_file_test (filename, G_FILE_TEST_EXISTS))
 	{
 	  cont = FALSE;
 	}
       else
 	{
 	  pixmap = gcompris_load_pixmap (str);
-	  
-	  imagelist = g_list_append (imagelist, pixmap);
+
+	  if(pixmap) {
+	    imagelist = g_list_append (imagelist, pixmap);
+	  } else {
+	    cont = FALSE;
+	  }
 	}
 
       g_free (filename);
