@@ -1,6 +1,6 @@
 /* gcompris - clockgame.c
  *
- * Time-stamp: <2001/09/12 23:44:20 bruno>
+ * Time-stamp: <2004/06/15 01:35:31 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -29,6 +29,7 @@
 static GList *item_list = NULL;
 
 static GcomprisBoard *gcomprisBoard = NULL;
+static gboolean board_paused = TRUE;
 
 static GnomeCanvasItem *second_item;
 static GnomeCanvasItem *hour_item;
@@ -48,30 +49,33 @@ typedef struct {
 } Time;
 static Time timeToFind, currentTime;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
-static void process_ok(void);
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void		 pause_board (gboolean pause);
+static void		 end_board (void);
+static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
+static void		 set_level (guint level);
+static void		 process_ok(void);
 
-static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent);
-static void clockgame_destroy_item(GnomeCanvasItem *item);
-static void clockgame_destroy_all_items(void);
-static void get_random_hour(Time *time);
-static void clockgame_next_level(void);
-static gint item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
-static void display_hour(guint hour);
-static void display_minute(guint minute);
-static void display_second(guint second);
+static int gamewon;
+static void		 game_won(void);
+
+static GnomeCanvasItem	*clockgame_create_item(GnomeCanvasGroup *parent);
+static void		 clockgame_destroy_item(GnomeCanvasItem *item);
+static void		 clockgame_destroy_all_items(void);
+static void		 get_random_hour(Time *time);
+static void		 clockgame_next_level(void);
+static gint		 item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
+static void		 display_hour(guint hour);
+static void		 display_minute(guint minute);
+static void		 display_second(guint second);
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
    N_("Learning Clock"),
-   N_("Learn how to read the time"),
+   N_("Learn how to tell time"),
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL,
    NULL,
@@ -84,6 +88,8 @@ BoardPlugin menu_bp =
    NULL,
    process_ok,
    set_level,
+   NULL,
+   NULL
 };
 
 
@@ -93,7 +99,7 @@ BoardPlugin menu_bp =
  *
  */
 
-BoardPlugin 
+BoardPlugin
 *get_bplugin_info(void)
 {
   return &menu_bp;
@@ -109,6 +115,12 @@ static void pause_board (gboolean pause)
   if(gcomprisBoard==NULL)
     return;
 
+  if(gamewon == TRUE && pause == FALSE) /* the game is won */
+    {
+      game_won();
+    }
+
+  board_paused = pause;
 }
 
 /*
@@ -120,25 +132,30 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "clockgame/clockgame-bg.jpg");
 
 
-      /* set initial values for this level */
+      /* set initial values for this level adjusted to fit the watch background */
       cx =  gcomprisBoard->width/2;
-      cy =  gcomprisBoard->height*0.4;
+      cy =  gcomprisBoard->height*0.4 + 42;
       clock_size = gcomprisBoard->height*0.3;
 
-      gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
       gcomprisBoard->level=1;
       gcomprisBoard->maxlevel=6;
-      gcomprisBoard->sublevel=0;
+      gcomprisBoard->sublevel=1;
       gcomprisBoard->number_of_sublevel=3; /* Go to next level after this number of 'play' */
+      gcompris_score_start(SCORESTYLE_NOTE,
+			   gcomprisBoard->width - 220,
+			   gcomprisBoard->height - 50,
+			   gcomprisBoard->number_of_sublevel);
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_OK);
-      gcompris_bar_set_timer(0);
 
       clockgame_next_level();
 
+      gamewon = FALSE;
       pause_board(FALSE);
+
+      gcompris_set_cursor(GCOMPRIS_LINE_CURSOR);
     }
 
 }
@@ -150,8 +167,11 @@ end_board ()
   if(gcomprisBoard!=NULL)
     {
       pause_board(TRUE);
+      gcompris_score_end();
       clockgame_destroy_all_items();
     }
+  gcomprisBoard = NULL;
+  gcompris_set_cursor(GCOMPRIS_DEFAULT_CURSOR);
 }
 
 static void
@@ -161,12 +181,12 @@ set_level (guint level)
   if(gcomprisBoard!=NULL)
     {
       gcomprisBoard->level=level;
-      gcomprisBoard->sublevel=0;
+      gcomprisBoard->sublevel=1;
       clockgame_next_level();
     }
 }
 
-gboolean
+static gboolean
 is_our_board (GcomprisBoard *gcomprisBoard)
 {
   if (gcomprisBoard)
@@ -189,11 +209,11 @@ is_our_board (GcomprisBoard *gcomprisBoard)
 /*-------------------------------------------------------------------------------*/
 
 /* set initial values for the next level */
-static void clockgame_next_level() 
+static void clockgame_next_level()
 {
 
   gcompris_bar_set_level(gcomprisBoard);
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcompris_score_set(gcomprisBoard->sublevel);
 
   clockgame_destroy_all_items();
 
@@ -217,7 +237,7 @@ static void clockgame_destroy_all_items()
 {
   GnomeCanvasItem *item;
 
-  while(g_list_length(item_list)>0) 
+  while(g_list_length(item_list)>0)
     {
       item = g_list_nth_data(item_list, 0);
       clockgame_destroy_item(item);
@@ -245,7 +265,7 @@ static void display_digital_time(GnomeCanvasItem *item, Time *time)
 
 static void display_hour(guint hour)
 {
-  double needle_size = clock_size*0.85;
+  double needle_size = clock_size*0.70;
   double ang;
   GnomeCanvasPoints *canvasPoints;
   canvasPoints = gnome_canvas_points_new (2);
@@ -256,7 +276,7 @@ static void display_hour(guint hour)
   /* Calc the needle angle */
   ang = ((hour > 12) ? hour-12 : hour) * M_PI / 6;
 
-  /* Depending on the level, work like a real clock, add the minute offset 
+  /* Depending on the level, work like a real clock, add the minute offset
      to the hour needle */
   if(gcomprisBoard->level>3)
     ang += currentTime.minute * M_PI / 360;
@@ -269,7 +289,7 @@ static void display_hour(guint hour)
 			 "points", canvasPoints,
 			 "fill_color", "darkblue",
 			 "width_units", (double)1,
-			 "width_pixels", (uint) 4,
+			 "width_pixels", (guint) 4,
 			 "last_arrowhead", TRUE,
 			 "arrow_shape_a", (double) needle_size,
 			 "arrow_shape_b", (double) needle_size-20,
@@ -299,9 +319,9 @@ static void display_minute(guint minute)
   canvasPoints->coords[3]=cy - needle_size * cos(ang);
   gnome_canvas_item_set (minute_item,
 			 "points", canvasPoints,
-			 "fill_color", "green",
+			 "fill_color", "red",
 			 "width_units", (double)1,
-			 "width_pixels", (uint) 4,
+			 "width_pixels", (guint) 4,
 			 "last_arrowhead", TRUE,
 			 "arrow_shape_a", (double) needle_size,
 			 "arrow_shape_b", (double) needle_size-10,
@@ -331,9 +351,9 @@ static void display_second(guint second)
   canvasPoints->coords[3]=cy - needle_size * cos(ang);
   gnome_canvas_item_set (second_item,
 			 "points", canvasPoints,
-			 "fill_color", "red",
+			 "fill_color_rgba", 0x68c46fFF,
 			 "width_units", (double)1,
-			 "width_pixels", (uint) 4,
+			 "width_pixels", (guint) 4,
 			 "last_arrowhead", TRUE,
 			 "arrow_shape_a", (double) 0,
 			 "arrow_shape_b", (double) 0,
@@ -356,15 +376,8 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
   GnomeCanvasPoints *canvasPoints;
   char *color;
   char *color_text;
-  GdkFont *font, *gdk_font, *gdk_font_small, *gdk_font_big, *gdk_font_big2, *gdk_font_big3;
   gchar *mtext = NULL;
-
-  /* Load a gdk font */
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-140-*-*-*-*-*-*");
-  gdk_font_small = gdk_font_load ("-adobe-times-medium-r-normal--*-120-*-*-*-*-*-*");
-  gdk_font_big = gdk_font_load ("-adobe-times-medium-r-normal--*-140-*-*-*-*-*-*");
-  gdk_font_big2 = gdk_font_load ("-adobe-times-medium-r-normal--*-180-*-*-*-*-*-*");
-  gdk_font_big3 = gdk_font_load ("-adobe-times-bold-r-normal--*-240-*-*-*-*-*-*");
+  gchar *font = NULL;
 
   canvasPoints = gnome_canvas_points_new (2);
 
@@ -377,17 +390,17 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 	  canvasPoints->coords[0]=cx + (needle_size-hour_point_size) * sin(ang);
 	  canvasPoints->coords[1]=cy - (needle_size-hour_point_size) * cos(ang);
 	  color="darkblue";
-	  color_text="green";
-	  font = gdk_font;
+	  color_text="red";
+	  font = gcompris_skin_font_board_small;
 	}
       else
 	{
 	  /* Minute point */
 	  canvasPoints->coords[0]=cx + (needle_size-min_point_size) * sin(ang);
 	  canvasPoints->coords[1]=cy - (needle_size-min_point_size) * cos(ang);
-	  color="green";
-	  color_text="lightgreen";
-	  font = gdk_font_small;
+	  color="red";
+	  color_text="red";
+	  font = gcompris_skin_font_board_tiny;
 	}
 
       canvasPoints->coords[2]=cx + needle_size * sin(ang);
@@ -397,7 +410,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 				    "points", canvasPoints,
 				    "fill_color", color,
 				    "width_units", (double)1,
-				    "width_pixels", (uint) 2,
+				    "width_pixels", (guint) 2,
 				    NULL);
       item_list = g_list_append (item_list, item);
 
@@ -408,7 +421,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 	  item = gnome_canvas_item_new (parent,
 					gnome_canvas_text_get_type (),
 					"text", mtext,
-					"font_gdk", font,
+					"font", font,
 					"x", (double) cx + (needle_size+10) * sin(ang),
 					"y", (double) cy - (needle_size+10) * cos(ang),
 					"anchor", GTK_ANCHOR_CENTER,
@@ -427,7 +440,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 	    item = gnome_canvas_item_new (parent,
 					  gnome_canvas_text_get_type (),
 					  "text", mtext,
-					  "font_gdk", gdk_font_big,
+					  "font", font,
 					  "x", (double) cx + (needle_size-30) * sin(ang),
 					  "y", (double) cy - (needle_size-30) * cos(ang),
 					  "anchor", GTK_ANCHOR_CENTER,
@@ -443,11 +456,11 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
   /* Create the text area for the digital time display */
   if(gcomprisBoard->level<4)
     {
-      digital_time_item = 
+      digital_time_item =
 	gnome_canvas_item_new (parent,
 			       gnome_canvas_text_get_type (),
 			       "text", "",
-			       "font_gdk", gdk_font_big2,
+			       "font", gcompris_skin_font_board_medium,
 			       "x", (double) cx,
 			       "y", (double) cy +  needle_size/2,
 			       "anchor", GTK_ANCHOR_CENTER,
@@ -472,7 +485,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 				     "points", canvasPoints,
 				     "fill_color", "darkblue",
 				     "width_units", (double)1,
-				     "width_pixels", (uint) 0,
+				     "width_pixels", (guint) 0,
 				     NULL);
   gtk_signal_connect(GTK_OBJECT(hour_item), "event",
 		     (GtkSignalFunc) item_event,
@@ -487,7 +500,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 				       "points", canvasPoints,
 				       "fill_color", "darkblue",
 				       "width_units", (double)1,
-				       "width_pixels", (uint) 0,
+				       "width_pixels", (guint) 0,
 				       NULL);
   gtk_signal_connect(GTK_OBJECT(minute_item), "event",
 		     (GtkSignalFunc) item_event,
@@ -502,7 +515,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 				       "points", canvasPoints,
 				       "fill_color", "darkblue",
 				       "width_units", (double)1,
-				       "width_pixels", (uint) 0,
+				       "width_pixels", (guint) 0,
 				       NULL);
   gtk_signal_connect(GTK_OBJECT(second_item), "event",
 		     (GtkSignalFunc) item_event,
@@ -511,31 +524,31 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
   display_second(currentTime.second);
 
   /* Create the text area for the time to find display */
-  item = 
+  item =
     gnome_canvas_item_new (parent,
 			   gnome_canvas_text_get_type (),
-			   "text", _("Set the clock to:"),
-			   "font_gdk", gdk_font_big3,
-			   "x", (double) cx,
-			   "y", (double) cy + needle_size +  needle_size / 3,
+			   "text", _("Set the watch to:"),
+			   "font", gcompris_skin_font_board_small,
+			   "x", (double) gcomprisBoard->width*0.15,
+			   "y", (double) cy + needle_size +  needle_size / 3 - 30,
 			   "anchor", GTK_ANCHOR_CENTER,
-			   "fill_color", "green",
+			   "fill_color", "white",
 			   NULL);
   item_list = g_list_append (item_list, item);
-  
-  time_to_find_item = 
+
+  time_to_find_item =
     gnome_canvas_item_new (parent,
 			   gnome_canvas_text_get_type (),
 			   "text", "",
-			   "font_gdk", gdk_font_big3,
-			   "x", (double) cx,
-			   "y", (double) cy + needle_size +  needle_size / 3 + 30,
+			   "font", gcompris_skin_font_board_big_bold,
+			   "x", (double) gcomprisBoard->width*0.15,
+			   "y", (double) cy + needle_size +  needle_size / 3,
 			   "anchor", GTK_ANCHOR_CENTER,
-			   "fill_color", "green",
+			   "fill_color", "white",
 			   NULL);
   display_digital_time(time_to_find_item, &timeToFind);
   item_list = g_list_append (item_list, time_to_find_item);
-  
+
   return (item);
 }
 
@@ -555,31 +568,38 @@ static gboolean time_equal(Time *time1, Time *time2)
 
 }
 
+/* ==================================== */
+static void game_won()
+{
+  gamewon = FALSE;
+  gcomprisBoard->sublevel++;
+
+  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel) {
+    /* Try the next level */
+    gcomprisBoard->sublevel=1;
+    gcomprisBoard->level++;
+    if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+      board_finished(BOARD_FINISHED_RANDOM);
+      return;
+    }
+    gcompris_play_ogg ("bonus", NULL);
+  }
+  clockgame_next_level();
+}
+
+/* ==================================== */
 static void process_ok()
 {
   if(time_equal(&timeToFind, &currentTime))
     {
-      gcompris_play_sound (SOUNDLISTFILE, "gobble");
-
-      gcomprisBoard->sublevel++;
-      gcompris_bar_set_timer(gcomprisBoard->sublevel);
-
-      if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
-	/* Try the next level */
-	gcomprisBoard->sublevel=0;
-	gcomprisBoard->level++;
-	if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-	    gcomprisBoard->level=gcomprisBoard->maxlevel;
-	gcompris_play_sound (SOUNDLISTFILE, "bonus");
-      }
-
-      clockgame_next_level();
+      gamewon = TRUE;
     }
   else
     {
       /* Oups, you're wrong */
-      gcompris_play_sound (SOUNDLISTFILE, "crash");
+      gamewon = FALSE;
     }
+  gcompris_display_bonus(gamewon, BONUS_FLOWER);
 }
 
 
@@ -593,28 +613,31 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
   GdkCursor *fleur;
   double new_x, new_y;
 
+  if(board_paused)
+    return FALSE;
+
   item_x = event->button.x;
   item_y = event->button.y;
   gnome_canvas_item_w2i(item->parent, &item_x, &item_y);
-  
-  switch (event->type) 
+
+  switch (event->type)
     {
     case GDK_BUTTON_PRESS:
       x = item_x;
       y = item_y;
-      
+
       fleur = gdk_cursor_new(GDK_FLEUR);
       gnome_canvas_item_grab(item,
-			     GDK_POINTER_MOTION_MASK | 
+			     GDK_POINTER_MOTION_MASK |
 			     GDK_BUTTON_RELEASE_MASK,
 			     fleur,
 			     event->button.time);
       gdk_cursor_destroy(fleur);
       dragging = TRUE;
       break;
-      
+
     case GDK_MOTION_NOTIFY:
-      if (dragging && (event->motion.state & GDK_BUTTON1_MASK)) 
+      if (dragging && (event->motion.state & GDK_BUTTON1_MASK))
 	{
 
           double angle;
@@ -625,7 +648,9 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
           new_y = - item_y + cy;
 
           /* angle as mesured relatively to noon position */
-          angle = M_PI/2 - atanf(new_y/new_x) + ((new_x<0) ? M_PI : 0.0);
+	  /* Thanks to Martin Herweg for this code        */
+	  angle =  atan2(new_x,new_y);
+	  if (angle<0) {angle= angle + 2.0*M_PI;}
 
 	  if(item==hour_item)
 	    display_hour(angle * 6 / M_PI);
@@ -638,15 +663,15 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
           y = new_y + cy;
 	}
       break;
-           
+
     case GDK_BUTTON_RELEASE:
-      if(dragging) 
+      if(dragging)
 	{
 	  gnome_canvas_item_ungrab(item, event->button.time);
 	  dragging = FALSE;
 	}
       break;
-      
+
     default:
       break;
     }
@@ -673,7 +698,7 @@ static void get_random_hour(Time *time)
     default:
       break;
     }
-  
+
 }
 
 
