@@ -1,6 +1,6 @@
 /* gcompris - paratrooper.c
  *
- * Time-stamp: <2002/04/14 00:25:03 bruno>
+ * Time-stamp: <2004/03/10 23:18:11 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -79,7 +79,7 @@ static void	 	 pause_board (gboolean pause);
 static void	 	 end_board (void);
 static gboolean	 	 is_our_board (GcomprisBoard *gcomprisBoard);
 static void	 	 set_level (guint level);
-gint		 	 key_press(guint keyval);
+static gint	 	 key_press(guint keyval);
 
 static GnomeCanvasItem	*paratrooper_create_cloud(GnomeCanvasGroup *parent);
 static gint		 paratrooper_drop_clouds (GtkWidget *widget, gpointer data);
@@ -97,12 +97,12 @@ static  double               speed = 0.0;
 static  double               imageZoom = 0.0;
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
    N_("Paratrooper"),
-   N_("Manage the paratrooper to let him safely land"),
+   N_("Direct the paratrooper to help him land safely"),
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL,
    NULL,
@@ -185,7 +185,7 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "images/scenery3_background.jpg");
 
 
       /* set initial values for this level */
@@ -228,7 +228,7 @@ set_level (guint level)
     }
 }
 
-gint key_press(guint keyval)
+static gint key_press(guint keyval)
 {
 
   if(!gcomprisBoard)
@@ -317,7 +317,7 @@ gint key_press(guint keyval)
   return TRUE;
 }
 
-gboolean
+static gboolean
 is_our_board (GcomprisBoard *gcomprisBoard)
 {
   if (gcomprisBoard)
@@ -637,7 +637,7 @@ static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
 	{
 	  gnome_canvas_item_hide(paratrooperItem.parachute);
 	  paratrooperItem.status = TUX_LANDED;
-	  gcompris_play_sound (SOUNDLISTFILE, "bonus");
+	  gcompris_play_ogg ("bonus", NULL);
 	  next_state();
 	}
       else
@@ -649,7 +649,7 @@ static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
 	    {
 	      gnome_canvas_item_hide(paratrooperItem.parachute);
 	      paratrooperItem.status = TUX_CRASHED;
-	      gcompris_play_sound (SOUNDLISTFILE, "crash");
+	      gcompris_play_ogg ("crash", NULL);
 	      next_state();
 	    }
 	}
