@@ -1,6 +1,6 @@
 /* gcompris - planegame.c
  *
- * Time-stamp: <2001/09/12 23:43:33 bruno>
+ * Time-stamp: <2004/03/08 00:01:15 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -35,6 +35,8 @@ static GnomeCanvasItem *planeitem = NULL;
 static gint plane_x, plane_y;
 static gint planespeed_x, planespeed_y;
 
+#define MAXSPEED 7
+
 /* These are the index for managing the game rule */
 static gint plane_target, plane_last_target;
 
@@ -52,7 +54,7 @@ static void pause_board (gboolean pause);
 static void end_board (void);
 static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
 static void set_level (guint level);
-gint key_press(guint keyval);
+static gint key_press(guint keyval);
 
 static GnomeCanvasItem *planegame_create_item(GnomeCanvasGroup *parent);
 static gint planegame_drop_items (GtkWidget *widget, gpointer data);
@@ -68,7 +70,7 @@ static  double               speed = 0.0;
 static  double               imageZoom = 0.0;
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
@@ -86,6 +88,8 @@ BoardPlugin menu_bp =
    key_press,
    NULL,
    set_level,
+   NULL,
+   NULL
 };
 
 /*
@@ -141,12 +145,12 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "images/scenery3_background.jpg");
 
 
       /* set initial values for this level */
       gcomprisBoard->level = 1;
-      gcomprisBoard->maxlevel = 6;
+      gcomprisBoard->maxlevel = 2;
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
       planegame_next_level();
@@ -164,9 +168,11 @@ end_board ()
   if(gcomprisBoard!=NULL)
     {
       pause_board(TRUE);
+      gcompris_score_end();
       planegame_destroy_all_items();
       gcomprisBoard->level = 1;       // Restart this game to zero
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -180,11 +186,11 @@ set_level (guint level)
     }
 }
 
-gint key_press(guint keyval)
+static gint key_press(guint keyval)
 {
 
-   if(!get_board_playing())
-     return FALSE;
+  if(!gcomprisBoard)
+    return TRUE;
 
   /* Add some filter for control and shift key */
   switch (keyval)
@@ -250,19 +256,19 @@ gint key_press(guint keyval)
       keyval=GDK_9;
       break;
     case GDK_Right:
-      if(planespeed_x < 3)
+      if(planespeed_x < MAXSPEED)
 	planespeed_x++;
       return TRUE;
     case GDK_Left:
-      if(planespeed_x > -3)
+      if(planespeed_x > -MAXSPEED)
 	planespeed_x--;
       return TRUE;
     case GDK_Up:
-      if(planespeed_y > -3)
+      if(planespeed_y > -MAXSPEED)
       planespeed_y--;
       return TRUE;
     case GDK_Down:
-      if(planespeed_y < 3)
+      if(planespeed_y < MAXSPEED)
       planespeed_y++;
       return TRUE;
     }
@@ -303,18 +309,14 @@ static void planegame_next_level()
 
   /* Try the next level */
   speed=100+(40/(gcomprisBoard->level));
-  fallSpeed=7000-gcomprisBoard->level*200;
-  gcomprisBoard->number_of_sublevel=10;
+  fallSpeed=10000-gcomprisBoard->level*200;
   /* Make the images tend to 0.5 ratio */
-  imageZoom=0.5+(0.5/(gcomprisBoard->level));
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  imageZoom=0.3+(0.5/(gcomprisBoard->level));
 
   /* Setup and Display the plane */
   planespeed_y = 0;
   planespeed_x = 0;
-  str = g_strdup_printf("%s%s", pixmapsdir, "tuxplane.png");
+  str = g_strdup_printf("%s%s", pixmapsdir, "tuxhelico.png");
   pixmap = gcompris_load_pixmap(str);
   plane_x = 50;
   plane_y = 300;
@@ -323,8 +325,8 @@ static void planegame_next_level()
 				     "pixbuf", pixmap, 
 				     "x", (double) plane_x,
 				     "y", (double) plane_y,
-				     "width", (double) gdk_pixbuf_get_width(pixmap)*imageZoom*0.7,
-				     "height", (double) gdk_pixbuf_get_height(pixmap)*imageZoom*0.7,
+				     "width", (double) gdk_pixbuf_get_width(pixmap)*imageZoom,
+				     "height", (double) gdk_pixbuf_get_height(pixmap)*imageZoom,
 				     "width_set", TRUE, 
 				     "height_set", TRUE,
 				     NULL);
@@ -332,22 +334,38 @@ static void planegame_next_level()
 
   /* Game rules */
   plane_target = 1;
-  plane_last_target = gcomprisBoard->level*10;
-  gcompris_bar_set_maxtimer(plane_last_target);
-  gcompris_bar_set_timer(plane_target-1);
+  plane_last_target = 10;
+
+  gcomprisBoard->number_of_sublevel=plane_last_target;
+
+  gcomprisBoard->sublevel=plane_target;
+
+  if(gcomprisBoard->level>1)
+    {
+      /* No scoring after level 1 */
+      gcompris_score_end();
+    }
+  else
+    {
+      gcompris_score_start(SCORESTYLE_NOTE, 
+			   gcomprisBoard->width - 220, 
+			   gcomprisBoard->height - 50, 
+			   gcomprisBoard->number_of_sublevel);
+      gcompris_score_set(gcomprisBoard->sublevel);
+    }
+
 
   g_free (str);
 
 }
 
+#define ISIN(x1, y1, px1, py1, px2, py2) (x1>px1 && x1<px2 && y1>py1 && y2<py2 ? TRUE : FALSE)
 
 static void planegame_cloud_colision(CloudItem *clouditem)
 {
   double px1, px2, py1, py2;
   double x1, x2, y1, y2;
   GnomeCanvasItem *item;
-  static CloudItem *inCloudItem = NULL;
-
 
   if(clouditem==NULL)
     return;
@@ -357,41 +375,36 @@ static void planegame_cloud_colision(CloudItem *clouditem)
   gnome_canvas_item_get_bounds(planeitem,  &px1, &py1, &px2, &py2); 
   gnome_canvas_item_get_bounds(item,  &x1, &y1, &x2, &y2); 
 
-  if(((px1>x1 && px1<x2) ||
-      (px2>x1 && px2<x2)) &&
-      (((py1>y1 && py1<y2) ||
-	(py2>y1 && py2<y2))))
+  if(
+     ISIN(x1, y1, px1, py1, px2, py2) ||
+     ISIN(x2, y1, px1, py1, px2, py2) ||
+     ISIN(x1, y2, px1, py1, px2, py2) ||
+     ISIN(x2, y2, px1, py1, px2, py2)
+     )
     {
       if(plane_target == clouditem->number)
 	{
-	  gcompris_play_sound (SOUNDLISTFILE, "gobble");
+	  gcompris_play_ogg ("gobble", NULL);
 	  item2del_list = g_list_append (item2del_list, clouditem);
 	  plane_target++;
-	  gcompris_bar_set_timer(plane_target-1);
-	  
+
+	  if(gcomprisBoard->level==1)
+	    {
+	      gcompris_score_set(plane_target);
+	    }
+
 	  if(plane_target==plane_last_target)
 	    {
 	      /* Try the next level */
 	      gcomprisBoard->level++;
-	      if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-		gcomprisBoard->level=gcomprisBoard->maxlevel;
+	      if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+		board_finished(BOARD_FINISHED_RANDOM);
+		return;
+	      }
 	      planegame_next_level();
-	      gcompris_play_sound (SOUNDLISTFILE, "bonus");
+	      gcompris_play_ogg ("bonus", NULL);
 	    }
 	}
-      else
-	{
-	  /* Oups, you get the wrong cloud */
-	  if(inCloudItem!=clouditem)
-	    gcompris_play_sound (SOUNDLISTFILE, "crash");
-	}
-      inCloudItem=clouditem;
-    }
-  else
-    {
-      /* OK, now we get out of the wrong cloud */
-      if(inCloudItem==clouditem)
-	inCloudItem=NULL;
     }
 }
 
@@ -489,7 +502,6 @@ static gint planegame_move_items (GtkWidget *widget, gpointer data)
 
   /* move the plane */
   planegame_move_plane(planeitem);
-
   dummy_id = gtk_timeout_add (speed,
 			      (GtkFunction) planegame_move_items, NULL);
 
@@ -498,7 +510,6 @@ static gint planegame_move_items (GtkWidget *widget, gpointer data)
 
 static GnomeCanvasItem *planegame_create_item(GnomeCanvasGroup *parent)
 {
-  GdkFont *gdk_font;
   GdkPixbuf *pixmap = NULL;
   GnomeCanvasItem *itemgroup;
   char *str = NULL;
@@ -506,8 +517,6 @@ static GnomeCanvasItem *planegame_create_item(GnomeCanvasGroup *parent)
   int i, min;
   CloudItem *clouditem;
 
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-240-*-*-*-*-*-*");
-
   /* Random cloud number */
   if(rand()%2==0)
     {
@@ -550,7 +559,7 @@ static GnomeCanvasItem *planegame_create_item(GnomeCanvasGroup *parent)
   gnome_canvas_item_new (GNOME_CANVAS_GROUP(itemgroup),
 			 gnome_canvas_text_get_type (),
 			 "text", number,
-			 "font_gdk", gdk_font,
+			 "font", gcompris_skin_font_board_big,
 			 "x", (double) 0,
 			 "y", (double) 0,
 			 "fill_color", "red",
@@ -585,7 +594,7 @@ static gint planegame_drop_items (GtkWidget *widget, gpointer data)
   planegame_add_new_item();
 
   drop_items_id = gtk_timeout_add (fallSpeed,
-				   (GtkFunction) planegame_drop_items, NULL);
+  				   (GtkFunction) planegame_drop_items, NULL);
   return (FALSE);
 }
 
