@@ -1,9 +1,9 @@
 /* gcompris - reading.c
  *
- * Time-stamp: <2001/09/12 23:45:21 bruno>
+ * Time-stamp: <2004/05/31 22:15:39 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
- * 
+ *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -19,7 +19,10 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+// FIXME: Cleanup of LettersItem created struct is not done
+
 #include <errno.h>
+#include <stdio.h>
 
 #include "gcompris/gcompris.h"
 
@@ -29,7 +32,8 @@
 
 static GcomprisBoard *gcomprisBoard = NULL;
 
-static gint drop_items_id = 0;
+static gint drop_items_id    = 0;
+static gint next_level_timer = 0;
 
 static char *textToFind = NULL;
 static gint textToFindIndex = 0;
@@ -37,52 +41,66 @@ static gint textToFindIndex = 0;
 
 static GnomeCanvasGroup *boardRootItem = NULL;
 
+typedef enum
+{
+  MODE_HORIZONTAL		= 0,
+  MODE_VERTICAL			= 1
+} Mode;
+static Mode currentMode = MODE_VERTICAL;
+
+/* Store the moving words */
 typedef struct {
-  char *word;
-  char *overword;
-  GnomeCanvasItem *rootitem;
-  GnomeCanvasItem *overwriteItem;
-  GnomeCanvasItem *item;
+  GnomeCanvasGroup *rootItem;
+  GnomeCanvasItem  *overwriteItem;
+  GnomeCanvasItem  *item;
 } LettersItem;
 
-#define BASE_X 220
-#define BASE_Y 120
+static LettersItem previousFocus;
+static LettersItem toDeleteFocus;
+
+
+/* Define the page area where text can be displayed */
+#define BASE_X1 70
+#define BASE_Y1 120
+#define BASE_X2 350
+#define BASE_Y2 520
+#define BASE_CX  BASE_X1+(BASE_X2-BASE_X1)/2
+
 gint current_x;
 gint current_y;
 gint numberOfLine;
 
-static LettersItem *currentFocus = NULL;
-static LettersItem *toHideItem = NULL;
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void		 pause_board (gboolean pause);
+static void		 end_board (void);
+static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
+static void		 set_level (guint level);
+static int		 wait_for_ready;
+static int		 gamewon;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
+static gboolean		 reading_create_item(GnomeCanvasGroup *parent);
+static gint		 reading_drop_items (void);
+//static void reading_destroy_item(LettersItem *item);
+static void		 reading_destroy_all_items(void);
+static gint		 reading_next_level(void);
 
-static GnomeCanvasItem *reading_create_item(GnomeCanvasGroup *parent);
-static gint reading_drop_items ();
-static void reading_destroy_item(LettersItem *item);
-static void reading_destroy_all_items(void);
-static void reading_next_level(void);
+static void		 player_win(void);
+static void		 player_loose(void);
+static char		*get_random_word();
+static GnomeCanvasItem	*display_what_to_do(GnomeCanvasGroup *parent);
+static void		 ask_ready(gboolean status);
+static void		 ask_yes_no(void);
+static gint		 item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
 
-static void player_win();
-static void player_loose(void);
-static char *get_random_word();
-static GnomeCanvasItem *display_what_to_do(GnomeCanvasGroup *parent);
-static void ask_ready(gboolean status);
-static void ask_yes_no();
-static gint item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
-
-static  guint32              fallSpeed = 0;
+static  guint32          fallSpeed = 0;
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
    N_("Reading"),
-   N_("Read a list of word and then tell if the given word is in it"),
+   N_("Read a list of words and then tell if the given word is in it"),
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL,
    NULL,
@@ -95,6 +113,8 @@ BoardPlugin menu_bp =
    NULL,
    NULL,
    set_level,
+   NULL,
+   NULL
 };
 
 /*
@@ -103,7 +123,7 @@ BoardPlugin menu_bp =
  *
  */
 
-BoardPlugin 
+BoardPlugin
 *get_bplugin_info(void)
 {
   return &menu_bp;
@@ -115,6 +135,10 @@ BoardPlugin
  */
 static void pause_board (gboolean pause)
 {
+  // after the bonus is ended, the board is unpaused, but we must wait for
+  // the player to be ready (this board does not use the same framework as others)
+  if (wait_for_ready)
+    return;
 
   if(gcomprisBoard==NULL)
     return;
@@ -144,12 +168,18 @@ static void start_board (GcomprisBoard *agcomprisBoard)
       gcomprisBoard=agcomprisBoard;
 
       gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "reading/gcompris-bg.jpg");
-
+      wait_for_ready = TRUE;
+      gamewon = FALSE;
 
       gcomprisBoard->level = 1;
       gcomprisBoard->maxlevel = 9;
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
+      /* Default mode */
+      currentMode=MODE_VERTICAL;
+      if(gcomprisBoard->mode && g_strcasecmp(gcomprisBoard->mode, "horizontal")==0)
+	currentMode=MODE_HORIZONTAL;
+
       reading_next_level();
     }
 }
@@ -163,6 +193,7 @@ end_board ()
       pause_board(TRUE);
       reading_destroy_all_items();
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -199,10 +230,12 @@ is_our_board (GcomprisBoard *gcomprisBoard)
 /*-------------------------------------------------------------------------------*/
 
 /* set initial values for the next level */
-static void reading_next_level() 
+static gint reading_next_level()
 {
   gcompris_bar_set_level(gcomprisBoard);
 
+  gamewon = FALSE;
+
   reading_destroy_all_items();
 
   boardRootItem = GNOME_CANVAS_GROUP(
@@ -214,67 +247,73 @@ static void reading_next_level()
 
   /* Default speed */
   fallSpeed=1400-gcomprisBoard->level*120;
-  numberOfLine=7+gcomprisBoard->level;
 
-  current_x = BASE_X;
-  current_y = BASE_Y;
+  if(currentMode==MODE_VERTICAL)
+    {
+      current_x = BASE_CX;
+      numberOfLine=7+gcomprisBoard->level;
+    }
+  else
+    {
+      current_x = BASE_X1;
+      numberOfLine=2+gcomprisBoard->level;
+    }
+
+  current_y = BASE_Y1;
 
   gcomprisBoard->number_of_sublevel=1;
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcomprisBoard->sublevel=1;
 
   display_what_to_do(boardRootItem);
   ask_ready(TRUE);
-}
 
-/*
- * cleanup
- */
-static void remove_old_word(LettersItem *item)
-{
-  /* The items are freed by player_win */
-  free(item->word);
-  free(item->overword);		  
-  free(item);
+  return (FALSE);
 }
 
 /* Destroy all the items */
 static void reading_destroy_all_items()
 {
-  LettersItem *item;
-
-  toHideItem = NULL;
-  currentFocus = NULL;
 
   if (drop_items_id) {
     gtk_timeout_remove (drop_items_id);
     drop_items_id = 0;
   }
 
+  if (next_level_timer) { 
+    gtk_timeout_remove (next_level_timer);
+    drop_items_id = 0;
+  }
+
   if(boardRootItem!=NULL)
       gtk_object_destroy (GTK_OBJECT(boardRootItem));
 
-  boardRootItem=NULL;
+  boardRootItem = NULL;
+  textToFind    = NULL;
+  previousFocus.rootItem = NULL;
+  toDeleteFocus.rootItem = NULL;
 }
 
 static GnomeCanvasItem *display_what_to_do(GnomeCanvasGroup *parent)
 {
-  GdkFont *gdk_font;
   gint base_Y = 110;
   gint base_X = 580;
   gint i;
 
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-240-*-*-*-*-*-*");
+  /* Free the previous text to find */
+  if(textToFind)
+    g_free(textToFind);
 
   /* Load the text to find */
   textToFind = "*";
   textToFind = get_random_word();
 
+  if(textToFind==NULL)
+    return;
+
   gnome_canvas_item_new (parent,
 			 gnome_canvas_text_get_type (),
-			 "text", _("Please, tell me if the word"),
-			 "font_gdk", gdk_font,
+			 "text", _("Please, check if the word"),
+			 "font", gcompris_skin_font_board_big,
 			 "x", (double) base_X,
 			 "y", (double) base_Y,
 			 "anchor", GTK_ANCHOR_CENTER,
@@ -284,7 +323,7 @@ static GnomeCanvasItem *display_what_to_do(GnomeCanvasGroup *parent)
   gnome_canvas_item_new (parent,
 			 gnome_canvas_text_get_type (),
 			 "text", textToFind,
-			 "font_gdk", gdk_font,
+			 "font", gcompris_skin_font_board_big,
 			 "x", (double) base_X,
 			 "y", (double) base_Y + 30,
 			 "anchor", GTK_ANCHOR_CENTER,
@@ -294,7 +333,7 @@ static GnomeCanvasItem *display_what_to_do(GnomeCanvasGroup *parent)
   gnome_canvas_item_new (parent,
 			 gnome_canvas_text_get_type (),
 			 "text", _("is being displayed"),
-			 "font_gdk", gdk_font,
+			 "font", gcompris_skin_font_board_big,
 			 "x", (double) base_X,
 			 "y", (double) base_Y + 60,
 			 "anchor", GTK_ANCHOR_CENTER,
@@ -312,94 +351,119 @@ static GnomeCanvasItem *display_what_to_do(GnomeCanvasGroup *parent)
   return NULL;
 }
 
-static GnomeCanvasItem *reading_create_item(GnomeCanvasGroup *parent)
+static gboolean reading_create_item(GnomeCanvasGroup *parent)
 {
-  LettersItem *lettersItem;
-  GdkFont *gdk_font;
-  gint i;
+  gint   i;
+  gint   anchor = GTK_ANCHOR_CENTER;
+  gchar *word;
+  gchar *overword;
 
-  if(toHideItem)
+  if(textToFind==NULL)
+    return FALSE;
+
+  if(toDeleteFocus.rootItem)
     {
-      gnome_canvas_item_hide(toHideItem->item);
-      gnome_canvas_item_hide(toHideItem->overwriteItem);
-      toHideItem = NULL;
+      gtk_object_destroy (GTK_OBJECT(toDeleteFocus.rootItem));
+      toDeleteFocus.rootItem = NULL;
     }
 
-  if(currentFocus)
+  if(previousFocus.rootItem)
     {
-      gnome_canvas_item_set (currentFocus->overwriteItem,
-			     "text", currentFocus->overword,
-			     NULL);
-      toHideItem = currentFocus;
+      gnome_canvas_item_show (previousFocus.overwriteItem);
+      toDeleteFocus.rootItem = previousFocus.rootItem;
     }
 
   if(numberOfLine<=0)
     {
-      gnome_canvas_item_hide(toHideItem->item);
-      gnome_canvas_item_hide(toHideItem->overwriteItem);
-      toHideItem = NULL;
+      gtk_object_destroy (GTK_OBJECT(toDeleteFocus.rootItem));
+      toDeleteFocus.rootItem = NULL;
+
       ask_yes_no();
-      return NULL;
+      return FALSE;
     }
 
-  lettersItem = malloc(sizeof(LettersItem));
-
-  /* Load a gdk font */
-  gdk_font = gdk_font_load ("-misc-fixed-medium-r-normal-*-*-200-*-*-c-*-iso8859-1");
-
-  
   if(textToFindIndex!=0)
     {
-      lettersItem->word = get_random_word();
+      word = get_random_word();
     }
   else
     {
-      lettersItem->word = textToFind;
+      word = g_strdup(textToFind);
     }
 
+  if(word==NULL)
+    return FALSE;
+
   if(textToFindIndex>=0)
     textToFindIndex--;
 
-  /* fill up the overword with zeros then with X */
-  lettersItem->overword=calloc(strlen(lettersItem->word), 1);
-  for(i=0; i<strlen(lettersItem->word); i++)
-    lettersItem->overword[i] = 'x';
+  /* fill up the overword with X */
+  overword = g_malloc(strlen(word));
+  for(i=0; i<strlen(word); i++)
+    overword[i] = 'x';
 
-  lettersItem->rootitem = \
-    gnome_canvas_item_new (parent,
-			   gnome_canvas_group_get_type (),
-			   "x", (double) current_x,
-			   "y", (double) current_y,
-			   NULL);
+  overword[i]='\0';
 
-  lettersItem->item = \
-    gnome_canvas_item_new (GNOME_CANVAS_GROUP(lettersItem->rootitem),
+  previousFocus.rootItem = \
+    GNOME_CANVAS_GROUP( gnome_canvas_item_new (parent,
+					       gnome_canvas_group_get_type (),
+					       "x", (double) current_x,
+					       "y", (double) current_y,
+					       NULL));
+
+  if(currentMode==MODE_HORIZONTAL)
+    anchor=GTK_ANCHOR_WEST;
+
+  previousFocus.item = \
+    gnome_canvas_item_new (GNOME_CANVAS_GROUP(previousFocus.rootItem),
 			   gnome_canvas_text_get_type (),
-			   "text", lettersItem->word,
-			   "font_gdk", gdk_font,
+			   "text", word,
+			   "font", gcompris_skin_font_board_fixed,
 			   "x", (double) 0,
 			   "y", (double) 0,
-			   "anchor", GTK_ANCHOR_CENTER,
+			   "anchor", anchor,
 			   "fill_color", "black",
 			   NULL);
+  g_free(word);
 
-  lettersItem->overwriteItem = \
-    gnome_canvas_item_new (GNOME_CANVAS_GROUP(lettersItem->rootitem),
+  previousFocus.overwriteItem = \
+    gnome_canvas_item_new (GNOME_CANVAS_GROUP(previousFocus.rootItem),
 			   gnome_canvas_text_get_type (),
-			   "text", "",
-			   "font_gdk", gdk_font,
+			   "text", overword,
+			   "font", gcompris_skin_font_board_fixed,
 			   "x", (double) 0,
 			   "y", (double) 0,
-			   "anchor", GTK_ANCHOR_CENTER,
+			   "anchor", anchor,
 			   "fill_color", "black",
 			   NULL);
+  gnome_canvas_item_hide(previousFocus.overwriteItem);
+  g_free(overword);
 
-  current_y += 20;
-  numberOfLine--;
+  // Calculate the next spot
+  if(currentMode==MODE_VERTICAL)
+    {
+      current_y += 20;
+      numberOfLine--;
+    }
+  else
+    {
+      double x1, y1, x2, y2;
 
-  currentFocus = lettersItem;
+      gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(previousFocus.rootItem), &x1, &y1, &x2, &y2);
 
-  return (lettersItem->rootitem);
+      // Are we out of bound
+      if(x2>BASE_X2)
+	{
+	  // Do the line Wrapping
+	  gnome_canvas_item_move(GNOME_CANVAS_ITEM(previousFocus.rootItem), BASE_X1-x1, 20);
+	  current_y += 20;
+	  current_x = BASE_X1;
+	  numberOfLine--;
+	}
+      current_x += x2-x1 + 12;
+    }
+
+  return (TRUE);
 }
 
 /*
@@ -422,7 +486,9 @@ static void ask_ready(gboolean status)
   GdkPixbuf *button_pixmap = NULL;
   double y_offset = 310;
   double x_offset = 430;
-  GdkFont *gdk_font;
+
+  if(textToFind==NULL)
+    return;
 
   if(status==FALSE)
     {
@@ -437,10 +503,8 @@ static void ask_ready(gboolean status)
       return;
     }
 
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-240-*-*-*-*-*-*");
-
   /*----- READY -----*/
-  button_pixmap = gcompris_load_pixmap("reading/button.png");
+  button_pixmap = gcompris_load_skin_pixmap("button_large2.png");
   item1 = gnome_canvas_item_new (boardRootItem,
 				gnome_canvas_pixbuf_get_type (),
 				"pixbuf",  button_pixmap, 
@@ -455,9 +519,9 @@ static void ask_ready(gboolean status)
 
   item2 = gnome_canvas_item_new (boardRootItem,
 				gnome_canvas_text_get_type (),
-				"text", _("I am Ready"), 
-				"font_gdk", gdk_font,
-				"x", (double) x_offset + 
+				"text", _("I am Ready"),
+				"font", gcompris_skin_font_board_big,
+				"x", (double) x_offset +
 				gdk_pixbuf_get_width(button_pixmap)/2,
 				"y", (double) y_offset + 40,
 				"anchor", GTK_ANCHOR_CENTER,
@@ -475,15 +539,15 @@ static void ask_yes_no()
   GdkPixbuf *button_pixmap = NULL;
   double y_offset = 310;
   double x_offset = 430;
-  GdkFont *gdk_font;
 
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-240-*-*-*-*-*-*");
+  if(textToFind==NULL)
+    return;
 
   /*----- YES -----*/
-  button_pixmap = gcompris_load_pixmap("reading/button.png");
+  button_pixmap = gcompris_load_skin_pixmap("button_large2.png");
   item = gnome_canvas_item_new (boardRootItem,
 				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
+				"pixbuf",  button_pixmap,
 				"x", x_offset,
 				"y", y_offset,
 				NULL);
@@ -495,9 +559,9 @@ static void ask_yes_no()
 
   item = gnome_canvas_item_new (boardRootItem,
 				gnome_canvas_text_get_type (),
-				"text", _("Yes I saw it"), 
-				"font_gdk", gdk_font,
-				"x", (double) x_offset + 
+				"text", _("Yes I saw it"),
+				"font", gcompris_skin_font_board_big,
+				"x", (double) x_offset +
 				gdk_pixbuf_get_width(button_pixmap)/2,
 				"y", (double) y_offset + 40,
 				"anchor", GTK_ANCHOR_CENTER,
@@ -510,10 +574,10 @@ static void ask_yes_no()
 
   /*----- NO -----*/
   y_offset += 100;
-  button_pixmap = gcompris_load_pixmap("reading/button.png");
+  button_pixmap = gcompris_load_skin_pixmap("button_large2.png");
   item = gnome_canvas_item_new (boardRootItem,
 				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
+				"pixbuf",  button_pixmap,
 				"x", x_offset,
 				"y", y_offset,
 				NULL);
@@ -525,9 +589,9 @@ static void ask_yes_no()
 
   item = gnome_canvas_item_new (boardRootItem,
 				gnome_canvas_text_get_type (),
-				"text", _("No, it was not there"), 
-				"font_gdk", gdk_font,
-				"x", (double) x_offset + 
+				"text", _("No, it was not there"),
+				"font", gcompris_skin_font_board_big,
+				"x", (double) x_offset +
 				gdk_pixbuf_get_width(button_pixmap)/2,
 				"y", (double) y_offset + 40,
 				"anchor", GTK_ANCHOR_CENTER,
@@ -542,25 +606,50 @@ static void ask_yes_no()
 
 static void player_win()
 {
-  gcompris_play_sound (SOUNDLISTFILE, "bonus");
+  gamewon = TRUE;
+  wait_for_ready = TRUE;
+  gcompris_display_bonus(gamewon, BONUS_FLOWER);
   /* Try the next level */
   gcomprisBoard->level++;
-  if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-    gcomprisBoard->level=gcomprisBoard->maxlevel;
-  reading_next_level();
+  if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+    board_finished(BOARD_FINISHED_RANDOM);
+    return;
+  }
+
+  next_level_timer = g_timeout_add(3000, (GtkFunction)reading_next_level, NULL);
 }
 
 static void player_loose()
 {
-  gcompris_play_sound (SOUNDLISTFILE, "crash");
-  reading_next_level();
+  gchar *expected;
+  gchar *got;
+  gamewon = FALSE;
+  wait_for_ready = TRUE;
+
+  /* Report what was wrong in the log */
+  expected = g_strdup_printf(_("The word to find was '%s'"), textToFind);
+
+  if(textToFindIndex == NOT_THERE)
+    got    = g_strdup_printf(_("But it was not displayed"));
+  else
+    got    = g_strdup_printf(_("And it was displayed"));
+
+  gcompris_log_set_comment (gcomprisBoard, expected, got);
+
+  g_free(expected);
+  g_free(got);
+
+  gcompris_display_bonus(gamewon, BONUS_FLOWER);
+
+  next_level_timer = g_timeout_add(3000, (GtkFunction)reading_next_level, NULL);
 }
 
 /* Callback for the yes and no buttons */
 static gint
 item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
 {
-  switch (event->type) 
+
+  switch (event->type)
     {
     case GDK_ENTER_NOTIFY:
       break;
@@ -570,20 +659,23 @@ item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
       if (((char *)data)[0]=='R')
 	{
 	  // The user is Ready
+	  wait_for_ready = FALSE;
 	  ask_ready(FALSE);
 	  pause_board(FALSE);
 	}
-      else if((((char *)data)[0]=='Y' && textToFindIndex == -1)
-	      || (((char *)data)[0]=='N' && textToFindIndex == NOT_THERE))
-	{
-	  player_win();
-	}
-      else
-	{
-	  player_loose();
-	} 
+      else if(!wait_for_ready) {
+	if ((((char *)data)[0]=='Y' && textToFindIndex == -1)
+	    || (((char *)data)[0]=='N' && textToFindIndex == NOT_THERE))
+	  {
+	    player_win();
+	  }
+	else
+	  {
+	    player_loose();
+	  }
+      }
       break;
-      
+
     default:
       break;
     }
@@ -592,7 +684,7 @@ item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
 }
 
 
-static FILE *get_wordfile(char *locale)
+static FILE *get_wordfile(const char *locale)
 {
   char *filename;
   FILE *wordsfd = NULL;
@@ -626,21 +718,13 @@ static FILE *get_wordfile(char *locale)
 static char *get_random_word()
 {
   FILE *wordsfd;
-  long size, i;
+  long  size, i;
   char *str;
+  int   try;
 
   str = malloc(MAXWORDSLENGTH);
 
-  if(!strcmp(setlocale(0, NULL), "C"))
-    {
-      /* NO Locale, Try use the English Locale */
-      wordsfd = get_wordfile("en");
-    } 
-  else 
-    {
-      wordsfd = get_wordfile(setlocale(0, NULL));
-    }
-
+  wordsfd = get_wordfile(gcompris_get_locale());
 
   if(wordsfd==NULL)
     {
@@ -648,35 +732,56 @@ static char *get_random_word()
       wordsfd = get_wordfile("en");
       
       /* Too bad, even english is not there. Check your Install */
-      if(wordsfd==NULL)
-	g_error("Cannot open file %s : Check your GCompris install", strerror(errno));
+      if(wordsfd==NULL) {
+	g_free(str);
+	gcompris_dialog(_("Cannot open file of words for your locale"), gcompris_end_board);
+	return NULL;
+      }
     }
 
   fseek (wordsfd, 0L, SEEK_END);
   size = ftell (wordsfd);
 
+  if(size==0) {
+    g_free(str);
+    gcompris_dialog(_("Cannot open file of words for your locale"), gcompris_end_board);
+    return NULL;
+  }
+
   i=rand()%size;
   fseek(wordsfd, i, SEEK_SET);
 
-  /* Read 2 times so that we are sure to sync on an end of line */
-  /* Warning, We are not allowed to return the text to find     */
-  fgets(str, MAXWORDSLENGTH, wordsfd);
+  try=-1;  /* We want to be sure to loop at least 2 times */
+  do {
 
-  if(ftell(wordsfd)==size || strncmp(textToFind, str, strlen(textToFind))==0)
-    rewind(wordsfd);
-
-  fgets(str, MAXWORDSLENGTH, wordsfd);
-
-  if(strncmp(textToFind, str, strlen(textToFind))==0)
+    /* Read 2 times so that we are sure to sync on an end of line */
+    /* Warning, We are not allowed to return the text to find     */
     fgets(str, MAXWORDSLENGTH, wordsfd);
 
-  /* Chop the return */
-  str[strlen(str)-1]='\0';
+    if(ftell(wordsfd)==size) {
+      rewind(wordsfd);
+      str[0]='\0';
+    }
+
+    /* Chop the return */
+    if(strlen(str)>0)
+      str[strlen(str)-1]='\0';
+
+    try++;
+
+  } while((str[0]=='\0' || strncmp(textToFind, str, strlen(textToFind))==0 || try<=0) && try<10);
+
+  /* We have a problem */
+  if(str[0]=='\0') {
+    gcompris_dialog(_("Error: The wordfile package used for your locale is corrupted."), gcompris_end_board);
+  }
 
   fclose(wordsfd);
 
-  if(strcmp(textToFind, str)==0)
+  if(strcmp(textToFind, str)==0) {
+    g_free(str);
     str=get_random_word();
+  }
 
   return (str);
 }
