@@ -1,6 +1,6 @@
 /* gcompris - smallnumbers.c
  *
- * Time-stamp: <2001/09/12 23:46:04 bruno>
+ * Time-stamp: <2004/03/08 00:02:14 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  * 
@@ -32,6 +32,7 @@ static gint dummy_id = 0;
 static gint drop_items_id = 0;
 
 static char *numbers = "123456789";
+static  int  gamewon;
 
 /* Hash table of all displayed letters  */
 static GHashTable *letters_table= NULL;
@@ -41,7 +42,7 @@ static void pause_board (gboolean pause);
 static void end_board (void);
 static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
 static void set_level (guint level);
-gint key_press(guint keyval);
+static gint key_press(guint keyval);
 
 static GnomeCanvasItem *smallnumbers_create_item(GnomeCanvasGroup *parent);
 static gint smallnumbers_drop_items (GtkWidget *widget, gpointer data);
@@ -62,12 +63,12 @@ static  double               speed = 0.0;
 static  double               imageZoom = 0.0;
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
    N_("Numbers With Dices"),
-   N_("Type the keyboard number 1 to 9 before the dices reach the ground"),
+   N_("Type the keyboard number 1 to 9 before the dice reaches the ground"),
    "Bruno Coudoin <bruno.coudoin@free.fr>",
    NULL,
    NULL,
@@ -80,6 +81,8 @@ BoardPlugin menu_bp =
    key_press,
    NULL,
    set_level,
+   NULL,
+   NULL
 };
 
 /*
@@ -117,6 +120,11 @@ static void pause_board (gboolean pause)
     }
   else
     {
+      if(gamewon == TRUE) /* the game is won */
+	{
+	  smallnumbers_next_level();
+	}
+
       if(!drop_items_id) {
 	drop_items_id = gtk_timeout_add (1000,
 					 (GtkFunction) smallnumbers_drop_items, NULL);
@@ -136,14 +144,20 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "images/scenery3_background.jpg");
 
       gcomprisBoard->level = 1;
       gcomprisBoard->maxlevel = 6;
+      gcomprisBoard->number_of_sublevel=10;
+      gcompris_score_start(SCORESTYLE_NOTE, 
+			   gcomprisBoard->width - 220, 
+			   gcomprisBoard->height - 50, 
+			   gcomprisBoard->number_of_sublevel);
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
       smallnumbers_next_level();
 
+      gamewon = FALSE;
       pause_board(FALSE);
     }
 }
@@ -154,8 +168,10 @@ end_board ()
   if(gcomprisBoard!=NULL)
     {
       pause_board(TRUE);
+      gcompris_score_end();
       smallnumbers_destroy_all_items();
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -169,14 +185,14 @@ set_level (guint level)
     }
 }
 
-gint key_press(guint keyval)
+static gint key_press(guint keyval)
 {
   gchar *old_value;
   gchar *old_name;
   char str[2];
 
-   if(!get_board_playing())
-     return FALSE;
+  if(!gcomprisBoard)
+    return TRUE;
 
   /* Add some filter for control and shift key */
   switch (keyval)
@@ -285,6 +301,7 @@ is_our_board (GcomprisBoard *gcomprisBoard)
 static void smallnumbers_next_level() 
 {
 
+  gamewon = FALSE;
   gcompris_bar_set_level(gcomprisBoard);
 
   smallnumbers_destroy_all_items();
@@ -292,11 +309,9 @@ static void smallnumbers_next_level()
   /* Try the next level */
   speed=100+(40/gcomprisBoard->level);
   fallSpeed=5000-gcomprisBoard->level*200;
-  gcomprisBoard->number_of_sublevel=10;
   imageZoom=0.9+(0.5/gcomprisBoard->level);
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcomprisBoard->sublevel=1;
+  gcompris_score_set(gcomprisBoard->sublevel);
 }
 
 
@@ -387,6 +402,8 @@ static GnomeCanvasItem *smallnumbers_create_item(GnomeCanvasGroup *parent)
   char *str;
   int i;
   char *lettersItem;
+  char *str1 = NULL;
+  char *str2 = NULL;
 
   if (!letters_table)
     {
@@ -405,6 +422,16 @@ static GnomeCanvasItem *smallnumbers_create_item(GnomeCanvasGroup *parent)
       return NULL;
     }
 
+  lettersItem[1] = '\0';
+
+  str1 = g_strdup_printf("%s%s", lettersItem, ".ogg");
+  str2 = gcompris_get_asset_file("gcompris alphabet", NULL, "audio/x-ogg", str1);
+
+  gcompris_play_ogg(str2, NULL);
+
+  g_free(str1);
+  g_free(str2);
+
   str = g_strdup_printf("gcompris/dice/gnome-dice%c.png", numbers[i]);
 
   smallnumbers_pixmap = gcompris_load_pixmap(str);
@@ -452,19 +479,23 @@ static gint smallnumbers_drop_items (GtkWidget *widget, gpointer data)
 static void player_win(GnomeCanvasItem *item)
 {
   smallnumbers_destroy_item(item);
-  gcompris_play_sound (SOUNDLISTFILE, "gobble");
+  gcompris_play_ogg ("gobble", NULL);
 
   gcomprisBoard->sublevel++;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcompris_score_set(gcomprisBoard->sublevel);
 
-  if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) 
+  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel)
     {
       /* Try the next level */
       gcomprisBoard->level++;
-      if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-	gcomprisBoard->level=gcomprisBoard->maxlevel;
-      smallnumbers_next_level();
-      gcompris_play_sound (SOUNDLISTFILE, "bonus");
+      if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+				gcompris_score_end();
+				board_finished(BOARD_FINISHED_RANDOM);
+				return;
+      }
+      gamewon = TRUE;
+      smallnumbers_destroy_all_items();
+      gcompris_display_bonus(gamewon, BONUS_SMILEY);
     }
   else
     {
@@ -486,7 +517,7 @@ static void player_win(GnomeCanvasItem *item)
 
 static void player_loose()
 {
-  gcompris_play_sound (SOUNDLISTFILE, "crash");
+  gcompris_play_ogg ("crash", NULL);
 }
 
 
