@@ -1,6 +1,6 @@
 /* gcompris - wordsgame.c
  *
- * Time-stamp: <2001/09/12 23:46:23 bruno>
+ * Time-stamp: <2004/06/12 23:45:50 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  * 
@@ -20,6 +20,7 @@
  */
 
 #include <errno.h>
+#include <stdio.h>
 
 #include "gcompris/gcompris.h"
 
@@ -39,43 +40,47 @@ static gint drop_items_id = 0;
 static GHashTable *words_table= NULL;
 
 typedef struct {
-  char *word;
-  char *overword;
+  gchar *word;
+  gchar *overword;
+  gint  charcounter;
   GnomeCanvasItem *rootitem;
   GnomeCanvasItem *overwriteItem;
 } LettersItem;
 
 static LettersItem *currentFocus = NULL;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
-gint key_press(guint keyval);
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void		 pause_board (gboolean pause);
+static void 		 end_board (void);
+static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
+static void		 set_level (guint level);
+static gint		 key_press(guint keyval);
 
-static GnomeCanvasItem *wordsgame_create_item(GnomeCanvasGroup *parent);
-static gint wordsgame_drop_items (GtkWidget *widget, gpointer data);
-static gint wordsgame_move_items (GtkWidget *widget, gpointer data);
-static void wordsgame_destroy_item(LettersItem *item);
-static void wordsgame_destroy_items(void);
-static void wordsgame_destroy_all_items(void);
-static void wordsgame_next_level(void);
-static void wordsgame_add_new_item(void);
+static GnomeCanvasItem	 *wordsgame_create_item(GnomeCanvasGroup *parent);
+static gint		 wordsgame_drop_items (GtkWidget *widget, gpointer data);
+static gint		 wordsgame_move_items (GtkWidget *widget, gpointer data);
+static void		 wordsgame_destroy_item(LettersItem *item);
+static void		 wordsgame_destroy_items(void);
+static void		 wordsgame_destroy_all_items(void);
+static void		 wordsgame_next_level(void);
+static void		 wordsgame_add_new_item(void);
 
-static void player_win(LettersItem *item);
-static void player_loose(void);
-static LettersItem *item_find_by_title (const gchar *title);
-static char *get_random_word();
-static void wordsgame_check_focus (char	*key,
+static void		 player_win(LettersItem *item);
+static void		 player_loose(void);
+static LettersItem 	*item_find_by_title (const gchar *title);
+static char		 *get_random_word(void);
+static void		 wordsgame_check_focus (gchar	*key,
 				   LettersItem *value,
 				   LettersItem **user_data);
+static gboolean words_table_foreach_remove (char *key,
+					    LettersItem *value,
+					    LettersItem *user_data);
 
 static  guint32              fallSpeed = 0;
 static  double               speed = 0.0;
 
 /* Description of this plugin */
-BoardPlugin menu_bp =
+static BoardPlugin menu_bp =
 {
    NULL,
    NULL,
@@ -93,6 +98,8 @@ BoardPlugin menu_bp =
    key_press,
    NULL,
    set_level,
+   NULL,
+   NULL
 };
 
 /*
@@ -131,11 +138,11 @@ static void pause_board (gboolean pause)
   else
     {
       if(!drop_items_id) {
-	drop_items_id = gtk_timeout_add (1000,
+	drop_items_id = gtk_timeout_add (100,
 					 (GtkFunction) wordsgame_drop_items, NULL);
       }
       if(!dummy_id) {
-	dummy_id = gtk_timeout_add (1000, (GtkFunction) wordsgame_move_items, NULL);
+	dummy_id = gtk_timeout_add (100, (GtkFunction) wordsgame_move_items, NULL);
       }
     }
 }
@@ -149,25 +156,24 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "images/scenery_background.jpg");
 
 
       gcomprisBoard->level = 1;
       gcomprisBoard->maxlevel = 6;
+      gcomprisBoard->sublevel = 1;
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
       wordsgame_next_level();
-
-      pause_board(FALSE);
     }
 }
 
-gboolean words_table_foreach_remove (char *key,
-				     LettersItem *value,
-				     LettersItem *user_data)
+static gboolean words_table_foreach_remove (char *key,
+					    LettersItem *value,
+					    LettersItem *user_data)
 {
-  free(value->word);
-  free(value->overword);
+  g_free(value->word);
+  g_free(value->overword);
   return TRUE;
 }
 
@@ -178,6 +184,7 @@ end_board ()
   if(gcomprisBoard!=NULL)
     {
       pause_board(TRUE);
+      gcompris_score_end();
       wordsgame_destroy_all_items();
       if (words_table)
 	{
@@ -188,6 +195,7 @@ end_board ()
 	  words_table=NULL;
 	}
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -201,16 +209,19 @@ set_level (guint level)
     }
 }
 
-static void wordsgame_check_focus (char	*key,
+static void wordsgame_check_focus (gchar       *key,
 				   LettersItem *value,
 				   LettersItem **user_data)
 {
   LettersItem *usrdata = *user_data;
+  gchar *nextCurrentChar;
 
   if(usrdata->rootitem!=NULL)
     return;
 
-  if(key[0]==usrdata->word[0])
+  nextCurrentChar = g_utf8_next_char(key);
+
+  if(strncmp(key, usrdata->word, nextCurrentChar-key)==0)
     {
       free(*user_data);
       *user_data = value;
@@ -218,12 +229,16 @@ static void wordsgame_check_focus (char	*key,
 
 }
 
-gint key_press(guint keyval)
+static gint key_press(guint keyval)
 {
-  char str[2];
+  char utf8char[6];
+  guint utf8_length;
 
-   if(!get_board_playing())
-     return FALSE;
+  if(!gcomprisBoard)
+    return TRUE;
+
+  if(!g_unichar_isalnum (gdk_keyval_to_unicode (keyval)))
+    return TRUE;
 
   /* Add some filter for control and shift key */
   switch (keyval)
@@ -278,7 +293,8 @@ gint key_press(guint keyval)
       break;
     }
 
-  sprintf(str, "%c", keyval);
+  utf8_length = g_unichar_to_utf8 (gdk_keyval_to_unicode(keyval),
+				   utf8char);
 
   if(currentFocus==NULL) 
     {
@@ -287,9 +303,10 @@ gint key_press(guint keyval)
       searchitem = malloc(sizeof(LettersItem));
 
       /* Try to see if this letter matches the first one of any words */
-      searchitem->word = (char *)&str;
-      searchitem->rootitem=NULL;
-      searchitem->overword="";
+      searchitem->word        = utf8char;
+      searchitem->rootitem    = NULL;
+      searchitem->overword    = "";
+      searchitem->charcounter = 0;
 
       g_hash_table_foreach (words_table, (GHFunc) wordsgame_check_focus, &searchitem);
 
@@ -308,26 +325,32 @@ gint key_press(guint keyval)
     {
       if(currentFocus->rootitem!=NULL) 
 	{
-	  char currentChar;
-	  
-	  /* Check this is the correct letter */
-	  currentChar = currentFocus->word[strlen(currentFocus->overword)];
+	  gchar *currentChar;
+	  gchar *nextCurrentChar;
+	  gint   i;
 
-	  if(currentChar==str[0])
+	  /* Find the next Char */
+	  currentChar = currentFocus->word + currentFocus->charcounter;
+
+	  if(strncmp(currentChar, utf8char, utf8_length)==0)
 	    {
+	      currentFocus->charcounter += utf8_length;
+
 	      /* Increment the overword */
-	      snprintf(currentFocus->overword, 
-		       strlen(currentFocus->overword)+2,
-		       "%s", currentFocus->word);
-	      
+	      strncpy(currentFocus->overword, currentFocus->word, currentFocus->charcounter);
+	      currentFocus->overword[currentFocus->charcounter] = '\0';
+
 	      gnome_canvas_item_set (currentFocus->overwriteItem,
 				     "text", currentFocus->overword,
 				     NULL);
-	      
-	      if(strlen(currentFocus->overword)==strlen(currentFocus->word))
+
+	      if(g_utf8_strlen(currentFocus->overword, MAXWORDSLENGTH) ==
+		 g_utf8_strlen(currentFocus->word, MAXWORDSLENGTH))
 		{
+		  LettersItem *item = item_find_by_title(currentFocus->word);
+
 		  /* You won Guy */
-		  player_win(item_find_by_title(currentFocus->word));
+		  player_win(item);
 
 		  currentFocus=NULL;
 		}
@@ -336,6 +359,7 @@ gint key_press(guint keyval)
 	    {
 	      /* It is a loose : unselect the word and defocus */
 	      currentFocus->overword[0]='\0';
+	      currentFocus->charcounter=0;
 	      gnome_canvas_item_set (currentFocus->overwriteItem,
 				     "text", currentFocus->overword,
 				     NULL);
@@ -350,11 +374,11 @@ gint key_press(guint keyval)
       /* Anyway kid you clicked on the wrong key */
       player_loose();
     }
-  
-  return TRUE;
+
+  return FALSE;
 }
 
-gboolean
+static gboolean
 is_our_board (GcomprisBoard *gcomprisBoard)
 {
   if (gcomprisBoard)
@@ -380,7 +404,15 @@ is_our_board (GcomprisBoard *gcomprisBoard)
 static void wordsgame_next_level() 
 {
 
+  gcomprisBoard->number_of_sublevel = 10 + 
+    ((gcomprisBoard->level-1) * 5);
+  gcompris_score_start(SCORESTYLE_NOTE, 
+		       gcomprisBoard->width - 220, 
+		       gcomprisBoard->height - 50, 
+		       gcomprisBoard->number_of_sublevel);
+  
   gcompris_bar_set_level(gcomprisBoard);
+  gcompris_score_set(gcomprisBoard->sublevel);
 
   wordsgame_destroy_all_items();
 
@@ -393,10 +425,8 @@ static void wordsgame_next_level()
     {
       fallSpeed=7000-gcomprisBoard->level*200;
     }
-  gcomprisBoard->number_of_sublevel=5;
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+
+  pause_board(FALSE);
 }
 
 static void remove_old_word(LettersItem *item)
@@ -404,9 +434,9 @@ static void remove_old_word(LettersItem *item)
   /* Remove old word */
   g_hash_table_remove (words_table, (item->word));
   /* The items are freed by player_win */
-  free(item->word);
-  free(item->overword);		  
-  free(item);
+  g_free(item->word);
+  g_free(item->overword);		  
+  g_free(item);
 }
 
 static void wordsgame_move_item(LettersItem *item)
@@ -430,6 +460,12 @@ static void wordsgame_move_item(LettersItem *item)
 static void wordsgame_destroy_item(LettersItem *item)
 {
 
+  /* Andrew Stribblehill <ads@debian.org> -- clear current focus if
+      it goes off the canvas */
+  if (currentFocus == item) {
+    currentFocus = NULL;
+  }
+
   item_list = g_list_remove (item_list, item);
   item2del_list = g_list_remove (item2del_list, item);
   gtk_object_destroy (GTK_OBJECT(item->rootitem));
@@ -484,7 +520,7 @@ static GnomeCanvasItem *wordsgame_create_item(GnomeCanvasGroup *parent)
 {
   GnomeCanvasItem *item2;
   LettersItem *lettersItem;
-  GdkFont *gdk_font;
+  guint maxtry = 10;
 
   lettersItem = malloc(sizeof(LettersItem));
 
@@ -493,26 +529,26 @@ static GnomeCanvasItem *wordsgame_create_item(GnomeCanvasGroup *parent)
       words_table= g_hash_table_new (g_str_hash, g_str_equal);
     }
 
-  /* Load a gdk font */
-  //  gdk_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-240-*-*-*-*-*-*");
-
   /* Beware, since we put the words in a hash table, we do not allow the same
      letter to be displayed two times */
+
   do {
     lettersItem->word = get_random_word();
-  } while(item_find_by_title(lettersItem->word)!=NULL);
+  } while(item_find_by_title(lettersItem->word)!=NULL && maxtry--);
+
+  if(maxtry==0) {
+    return NULL;
+  }
 
   /* fill up the overword with zeros */
   lettersItem->overword=calloc(strlen(lettersItem->word), 1);
+  lettersItem->charcounter=0;
 
   lettersItem->rootitem = \
     gnome_canvas_item_new (parent,
 			   gnome_canvas_group_get_type (),
-			   "x", (double)(rand()%(gcomprisBoard->width-
-						 (guint)(gdk_string_width(gdk_font, 
-									  lettersItem->word)))),
-			   "y", (double) -gdk_string_height(gdk_font, lettersItem->word),
+			   "x", (double)(rand()%(gcomprisBoard->width-170)),
+			   "y", (double) -12,
 			   NULL);
 
   /* To 'erase' words, I create 2 times the text item. One is empty now */
@@ -521,22 +557,22 @@ static GnomeCanvasItem *wordsgame_create_item(GnomeCanvasGroup *parent)
     gnome_canvas_item_new (GNOME_CANVAS_GROUP(lettersItem->rootitem),
 			   gnome_canvas_text_get_type (),
 			   "text", lettersItem->word,
-			   "font_gdk", gdk_font,
+			   "font", gcompris_skin_font_board_huge_bold,
 			   "x", (double) 0,
 			   "y", (double) 0,
 			   "anchor", GTK_ANCHOR_NW,
-			   "fill_color", "white",
+			   "fill_color", "black",
 			   NULL);
 
   lettersItem->overwriteItem = \
     gnome_canvas_item_new (GNOME_CANVAS_GROUP(lettersItem->rootitem),
 			   gnome_canvas_text_get_type (),
 			   "text", "",
-			   "font_gdk", gdk_font,
+			   "font", gcompris_skin_font_board_huge_bold,
 			   "x", (double) 0,
 			   "y", (double) 0,
 			   "anchor", GTK_ANCHOR_NW,
-			   "fill_color", "green",
+			   "fill_color", "blue",
 			   NULL);
 
   item_list = g_list_append (item_list, lettersItem);
@@ -568,20 +604,24 @@ static gint wordsgame_drop_items (GtkWidget *widget, gpointer data)
 static void player_win(LettersItem *item)
 {
 
-  wordsgame_destroy_item(item);
-  gcompris_play_sound (SOUNDLISTFILE, "gobble");
+  gcompris_play_ogg ("gobble", NULL);
 
   gcomprisBoard->sublevel++;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcompris_score_set(gcomprisBoard->sublevel);
 
-  if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) 
+  wordsgame_destroy_item(item);
+
+  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel) 
     {
       /* Try the next level */
       gcomprisBoard->level++;
-      if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-	gcomprisBoard->level=gcomprisBoard->maxlevel;
+      gcomprisBoard->sublevel = 1;
+      if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+	board_finished(BOARD_FINISHED_RANDOM);
+	return;
+      }
       wordsgame_next_level();
-      gcompris_play_sound (SOUNDLISTFILE, "bonus");
+      gcompris_play_ogg ("bonus", NULL);
     }
   else
     {
@@ -603,7 +643,7 @@ static void player_win(LettersItem *item)
 
 static void player_loose()
 {
-  gcompris_play_sound (SOUNDLISTFILE, "crash");
+  gcompris_play_ogg ("crash", NULL);
 }
 
 static LettersItem *
@@ -646,25 +686,15 @@ static FILE *get_wordfile(char *locale)
  * Return a random word from a set of text file depending on 
  * the current level and language
  */
-static char *get_random_word()
+static gchar *get_random_word()
 {
   FILE *wordsfd;
   long size, i;
-  char *str;
+  gchar *str;
 
-  str = malloc(MAXWORDSLENGTH);
-
-  printf("Locale returned by setlocale(0, NULL) : %s\n",setlocale(0, NULL));
-  if(!strcmp(setlocale(0, NULL), "C"))
-    {
-      /* NO Locale, Try use the English Locale */
-      wordsfd = get_wordfile("en");
-    } 
-  else 
-    {
-      wordsfd = get_wordfile(setlocale(0, NULL));
-    }
+  str = g_malloc(MAXWORDSLENGTH);
 
+  wordsfd = get_wordfile(gcompris_get_locale());
 
   if(wordsfd==NULL)
     {
