@@ -1,6 +1,6 @@
 /* gcompris - properties.h
  *
- * Time-stamp: <2002/04/14 01:10:37 bruno>
+ * Time-stamp: <2004/06/05 00:15:38 bcoudoin>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -25,17 +25,43 @@
 typedef struct {
   gint		music;
   gint		fx;
+  gint		audio_works;
   gint		fullscreen;
+  gint		screensize;
   gint		defaultcursor;
+  gint		timer;
+  gint		filter_style;
+  gint		difficulty_filter;
+  gint		difficulty_max;
   gchar	       *locale;
+  gchar	       *skin;
+  gchar	       *key;
 } GcomprisProperties;
 
 GcomprisProperties	*gcompris_get_properties (void);
 GcomprisProperties	*gcompris_properties_new (void);
 void			 gcompris_properties_destroy (GcomprisProperties *props);
-GcomprisProperties	*gcompris_properties_copy (GcomprisProperties *props);
 void			 gcompris_properties_save (GcomprisProperties *props);
 
+void			 gcompris_properties_enable_board(gchar *boardName);
+void			 gcompris_properties_disable_board(gchar *boardName);
+gboolean		 gcompris_properties_get_board_status(gchar *boardName);
+void			 gcompris_write_boards_status();
+
+/* private definitions */
+typedef enum {
+  GCOMPRIS_PROP_MUSIC,
+  GCOMPRIS_PROP_FX,
+  GCOMPRIS_PROP_SCREENSIZE,
+  GCOMPRIS_PROP_TIMER,
+  GCOMPRIS_PROP_FULLSCREEN,
+  GCOMPRIS_PROP_LOCALE,
+  GCOMPRIS_PROP_SKIN,
+  GCOMPRIS_PROP_AUDIO,
+  GCOMPRIS_PROP_DIFFICULTY,
+  GCOMPRIS_PROP_FILTER_STYLE
+} GcomprisTokenType;
+
 #endif
 
 /* Local Variables: */
